#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to tasks."""

from opencue_proto import task_pb2
from opencue.cuebot import Cuebot


class Task(object):
    """This class contains the grpc implementation related to a Task."""

    def __init__(self, task=None):
        self.data = task
        self.stub = Cuebot.getStub('task')

    def id(self):
        """Returns the unique id of the task.

        :rtype: str
        :return: task id
        """
        return self.data.id

    def setMinCores(self, minCores):
        """Sets the minimum amount of cores for the task.

        :type  minCores: int
        :param minCores: the minimum number of cores the task needs
        """
        self.stub.SetMinCores(
            task_pb2.TaskSetMinCoresRequest(task=self.data, new_min_cores=minCores),
            timeout=Cuebot.Timeout)

    def delete(self):
        """Deletes the task."""
        self.stub.Delete(task_pb2.TaskDeleteRequest(task=self.data), timeout=Cuebot.Timeout)



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to deeds."""

from opencue_proto import host_pb2
from opencue.cuebot import Cuebot
# pylint: disable=cyclic-import
import opencue.wrappers.host
import opencue.wrappers.owner


class Deed(object):
    """This class contains the grpc implementation related to a Deed."""

    def __init__(self, deed=None):
        self.data = deed
        self.stub = Cuebot.getStub('comment')

    def delete(self):
        """Deletes the deed."""
        self.stub.Delete(host_pb2.DeedDeleteRequest(deed=self.data), timeout=Cuebot.Timeout)

    def getHost(self):
        """Returns the host for the deed.

        :rtype:  opencue.wrappers.host.Host
        :return: deed host
        """
        return opencue.wrappers.host.Host(
            self.stub.GetHost(host_pb2.DeedGetHostRequest(deed=self.data),
                              timeout=Cuebot.Timeout).host)

    def getOwner(self):
        """Returns the owner of the deed.

        :rtype:  opencue.wrappers.owner.Owner
        :return: deed owner
        """
        return opencue.wrappers.owner.Owner(
            self.stub.GetOwner(host_pb2.DeedGetOwnerRequest(deed=self.data),
                               timeout=Cuebot.Timeout).owner)

    def id(self):
        """Returns the id of the deed.

        :rtype:  str
        :return: deed id
        """
        return self.data.id

    def host(self):
        """Returns the name of the host associated with the deed.

        :rtype:  str
        :return: name of the deed host
        """
        return self.data.host

    def owner(self):
        """Returns the name of the owner of the deed.

        :rtype:  str
        :return: name of the deed owner
        """
        return self.data.owner

    def show(self):
        """Returns the name of the show of the deed.

        :rtype:  str
        :return: name of the deed show
        """
        return self.data.show



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.util`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

import os
import time
import unittest

import opencue.wrappers.util


TEST_SECONDS_A = 1557942905
TEST_SECONDS_B = 2000
TEST_SECONDS_C = 0
MEM_500_M = 1024 * 512
MEM_6_G = 1024 * 1024 * 6
MEM_6000_G = 1024 * 1024 * 1024 * 6


class UtilTests(unittest.TestCase):

    def setUp(self):
        os.environ['TZ'] = 'Europe/London'
        time.tzset()

    def testFormatTime(self):
        expected = '05/15 18:55'
        timeString = opencue.wrappers.util.format_time(TEST_SECONDS_A)
        self.assertEqual(timeString, expected)

        expected = '01/01 01:33'
        timeString = opencue.wrappers.util.format_time(TEST_SECONDS_B)
        self.assertEqual(timeString, expected)

        expected = '--/-- --:--'
        timeString = opencue.wrappers.util.format_time(TEST_SECONDS_C)
        self.assertEqual(timeString, expected)


    def testDateToMMDDHHMM(self):
        expected = '05/15 18:55'
        timeString = opencue.wrappers.util.dateToMMDDHHMM(TEST_SECONDS_A)
        self.assertEqual(timeString, expected)

        expected = '01/01 01:33'
        timeString = opencue.wrappers.util.dateToMMDDHHMM(TEST_SECONDS_B)
        self.assertEqual(timeString, expected)

        expected = '--/-- --:--'
        timeString = opencue.wrappers.util.dateToMMDDHHMM(TEST_SECONDS_C)
        self.assertEqual(timeString, expected)

    def testSecondsToHHMMSS(self):
        expected = '432761:55:05'
        timeString = opencue.wrappers.util.secondsToHHMMSS(TEST_SECONDS_A)
        self.assertEqual(timeString, expected)

        expected = '00:33:20'
        timeString = opencue.wrappers.util.secondsToHHMMSS(TEST_SECONDS_B)
        self.assertEqual(timeString, expected)

        expected = '00:00:00'
        timeString = opencue.wrappers.util.secondsToHHMMSS(TEST_SECONDS_C)
        self.assertEqual(timeString, expected)

    def testSecondsToHMMSS(self):
        expected = '432761:55:05'
        timeString = opencue.wrappers.util.secondsToHMMSS(TEST_SECONDS_A)
        self.assertEqual(timeString, expected)

        expected = '0:33:20'
        timeString = opencue.wrappers.util.secondsToHMMSS(TEST_SECONDS_B)
        self.assertEqual(timeString, expected)

        expected = '0:00:00'
        timeString = opencue.wrappers.util.secondsToHMMSS(TEST_SECONDS_C)
        self.assertEqual(timeString, expected)

    def testSecondsToHHHMM(self):
        expected = '432761:55'
        timeString = opencue.wrappers.util.secondsToHHHMM(TEST_SECONDS_A)
        self.assertEqual(timeString, expected)

        expected = '000:33'
        timeString = opencue.wrappers.util.secondsToHHHMM(TEST_SECONDS_B)
        self.assertEqual(timeString, expected)

        expected = '000:00'
        timeString = opencue.wrappers.util.secondsToHHHMM(TEST_SECONDS_C)
        self.assertEqual(timeString, expected)

    def testSecondsDiffToHMMSS(self):

        expected = '432761:21:45'
        timeString = opencue.wrappers.util.secondsDiffToHMMSS(TEST_SECONDS_B, TEST_SECONDS_A)
        self.assertEqual(timeString, expected)

        expected = '432761:21:45'
        timeString = opencue.wrappers.util.secondsDiffToHMMSS(TEST_SECONDS_A, TEST_SECONDS_B)
        self.assertEqual(timeString, expected)

    def testConvertMem(self):
        expected = '6291456K'
        memString = opencue.wrappers.util.convert_mem(MEM_6_G, 'K')
        self.assertEqual(memString, expected)

        expected = '6291456M'
        memString = opencue.wrappers.util.convert_mem(MEM_6000_G, 'M')
        self.assertEqual(memString, expected)

        expected = '6.0G'
        memString = opencue.wrappers.util.convert_mem(MEM_6_G, 'G')
        self.assertEqual(memString, expected)

        expected = '0.5G'
        memString = opencue.wrappers.util.convert_mem(MEM_500_M, 'G')
        self.assertEqual(memString, expected)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.api`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import cue_pb2
from opencue_proto import depend_pb2
from opencue_proto import facility_pb2
from opencue_proto import filter_pb2
from opencue_proto import host_pb2
from opencue_proto import job_pb2
from opencue_proto import limit_pb2
from opencue_proto import service_pb2
from opencue_proto import show_pb2
from opencue_proto import subscription_pb2
import opencue.api


TEST_SHOW_NAME = 'pipe'
TEST_GROUP_NAME = 'pipe'
TEST_GROUP_ID = 'A0000000-0000-0000-0000-000000000000'
TEST_JOB_NAME = 'pipe-dev.cue-chambers_shell_v6'
TEST_LAYER_NAME = 'depend_er'
TEST_LIMIT_NAME = 'test-limit'
TEST_HOST_NAME = 'wolf1001'
TEST_SUB_NAME = 'pipe.General'
TEST_FACILITY_NAME = 'arbitrary-facility-name'
TEST_TAG = 'General'
TEST_ALLOC_NAME = 'pipe.General'
TEST_PROC_NAME = 'arbitrary-proc-name'



class ShowTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetShows(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetShows.return_value = show_pb2.ShowGetShowsResponse(
            shows=show_pb2.ShowSeq(shows=[show_pb2.Show(name=TEST_SHOW_NAME)]))
        getStubMock.return_value = stubMock

        showList = opencue.api.getShows()

        stubMock.GetShows.assert_called_with(show_pb2.ShowGetShowsRequest(), timeout=mock.ANY)
        self.assertEqual([TEST_SHOW_NAME], [show.name() for show in showList])

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindShow(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.FindShow.return_value = show_pb2.ShowFindShowResponse(
            show=show_pb2.Show(name=TEST_SHOW_NAME))
        getStubMock.return_value = stubMock

        show = opencue.api.findShow(TEST_SHOW_NAME)

        stubMock.FindShow.assert_called_with(
            show_pb2.ShowFindShowRequest(name=TEST_SHOW_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_SHOW_NAME, show.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testCreateShow(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.CreateShow.return_value = show_pb2.ShowCreateShowResponse(
            show=show_pb2.Show(name=TEST_SHOW_NAME))
        getStubMock.return_value = stubMock

        newShow = opencue.api.createShow(TEST_SHOW_NAME)

        stubMock.CreateShow.assert_called_with(
            show_pb2.ShowCreateShowRequest(name=TEST_SHOW_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_SHOW_NAME, newShow.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testDeleteShow(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        showToDelete = show_pb2.Show(id=arbitraryId)
        stubMock = mock.Mock()
        stubMock.FindShow.return_value = show_pb2.ShowFindShowResponse(show=showToDelete)
        stubMock.Delete.return_value = show_pb2.ShowDeleteResponse()
        getStubMock.return_value = stubMock

        opencue.api.deleteShow(arbitraryId)

        stubMock.Delete.assert_called_with(
            show_pb2.ShowDeleteRequest(show=showToDelete), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetActiveShows(self, getStubMock):
        show1Name = 'first-show'
        show2Name = 'second-show'
        stubMock = mock.Mock()
        stubMock.GetActiveShows.return_value = show_pb2.ShowGetActiveShowsResponse(
            shows=show_pb2.ShowSeq(
                shows=[show_pb2.Show(name=show1Name), show_pb2.Show(name=show2Name)]))
        getStubMock.return_value = stubMock

        showList = opencue.api.getActiveShows()

        stubMock.GetActiveShows.assert_called_with(
            show_pb2.ShowGetActiveShowsRequest(), timeout=mock.ANY)
        self.assertEqual([show1Name, show2Name], [show.name() for show in showList])


class GroupTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindGroup(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.FindGroup.return_value = job_pb2.GroupFindGroupResponse(
            group=job_pb2.Group(name=TEST_GROUP_NAME))
        getStubMock.return_value = stubMock

        group = opencue.api.findGroup(TEST_SHOW_NAME, TEST_GROUP_NAME)

        stubMock.FindGroup.assert_called_with(
            job_pb2.GroupFindGroupRequest(
                show=TEST_SHOW_NAME, name=TEST_GROUP_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_GROUP_NAME, group.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetGroup(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetGroup.return_value = job_pb2.GroupGetGroupResponse(
            group=job_pb2.Group(id=TEST_GROUP_ID))
        getStubMock.return_value = stubMock

        group = opencue.api.getGroup(TEST_GROUP_ID)

        stubMock.GetGroup.assert_called_with(
            job_pb2.GroupGetGroupRequest(id=TEST_GROUP_ID), timeout=mock.ANY)
        self.assertEqual(TEST_GROUP_ID, group.id())


class JobTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testIsJobPending(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.IsJobPending.return_value = job_pb2.JobIsJobPendingResponse(value=True)
        getStubMock.return_value = stubMock

        self.assertTrue(opencue.api.isJobPending(TEST_SHOW_NAME))

        stubMock.IsJobPending.return_value = job_pb2.JobIsJobPendingResponse(value=False)

        self.assertFalse(opencue.api.isJobPending(TEST_SHOW_NAME))

        stubMock.IsJobPending.assert_called_with(
            job_pb2.JobIsJobPendingRequest(name=TEST_SHOW_NAME), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindJob(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.FindJob.return_value = job_pb2.JobFindJobResponse(
            job=job_pb2.Job(name=TEST_JOB_NAME))
        getStubMock.return_value = stubMock

        job = opencue.api.findJob(TEST_JOB_NAME)

        stubMock.FindJob.assert_called_with(
            job_pb2.JobFindJobRequest(name=TEST_JOB_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_JOB_NAME, job.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetJobs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetJobs.return_value = job_pb2.JobGetJobsResponse(
            jobs=job_pb2.JobSeq(jobs=[job_pb2.Job(name=TEST_JOB_NAME)]))
        getStubMock.return_value = stubMock

        jobsByShow = opencue.api.getJobs(show=[TEST_SHOW_NAME])

        stubMock.GetJobs.assert_called_with(
            job_pb2.JobGetJobsRequest(
                r=job_pb2.JobSearchCriteria(shows=[TEST_SHOW_NAME])), timeout=mock.ANY)
        self.assertEqual(1, len(jobsByShow))
        self.assertEqual(TEST_JOB_NAME, jobsByShow[0].name())

        jobsByName = opencue.api.getJobs(name=[TEST_JOB_NAME], show=[TEST_SHOW_NAME])

        stubMock.GetJobs.assert_called_with(
            job_pb2.JobGetJobsRequest(
                r=job_pb2.JobSearchCriteria(
                    jobs=[TEST_JOB_NAME], shows=[TEST_SHOW_NAME])), timeout=mock.ANY)
        self.assertEqual(1, len(jobsByName))
        self.assertEqual(TEST_JOB_NAME, jobsByName[0].name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetAllJobs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetJobs.return_value = job_pb2.JobGetJobsResponse(
            jobs=job_pb2.JobSeq(jobs=[job_pb2.Job(name=TEST_JOB_NAME)]))
        getStubMock.return_value = stubMock

        jobs = opencue.api.getJobs()

        stubMock.GetJobs.assert_called_with(
            job_pb2.JobGetJobsRequest(
                r=job_pb2.JobSearchCriteria()), timeout=mock.ANY)
        self.assertEqual(1, len(jobs))
        self.assertEqual(TEST_JOB_NAME, jobs[0].name())

    def testRaiseExceptionOnBadCriteriaSearch(self):
        with self.assertRaises(Exception) as context:
            opencue.api.getJobs(bad_criteria=["00000000-0000-0000-0000-012345678980"])

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetJob(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.GetJob.return_value = job_pb2.JobGetJobResponse(job=job_pb2.Job(id=arbitraryId))
        getStubMock.return_value = stubMock

        job = opencue.api.getJob(arbitraryId)

        stubMock.GetJob.assert_called_with(
            job_pb2.JobGetJobRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, job.id())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetJobNames(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetJobNames.return_value = job_pb2.JobGetJobNamesResponse(
            names=[TEST_JOB_NAME])
        getStubMock.return_value = stubMock

        jobNames = opencue.api.getJobNames(show=[TEST_SHOW_NAME])

        stubMock.GetJobNames.assert_called_with(
            job_pb2.JobGetJobNamesRequest(
                r=job_pb2.JobSearchCriteria(shows=[TEST_SHOW_NAME])), timeout=mock.ANY)
        self.assertEqual(1, len(jobNames))
        self.assertEqual(TEST_JOB_NAME, jobNames[0])

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testLaunchSpec(self, getStubMock):
        spec = 'arbitrary-spec'
        stubMock = mock.Mock()
        stubMock.LaunchSpec.return_value = job_pb2.JobLaunchSpecResponse(
            names=[TEST_JOB_NAME])
        getStubMock.return_value = stubMock

        jobNames = opencue.api.launchSpec(spec)

        stubMock.LaunchSpec.assert_called_with(
            job_pb2.JobLaunchSpecRequest(spec=spec), timeout=mock.ANY)
        self.assertEqual([TEST_JOB_NAME], jobNames)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testLaunchSpecAndWait(self, getStubMock):
        spec = 'arbitrary-spec'
        stubMock = mock.Mock()
        stubMock.LaunchSpecAndWait.return_value = job_pb2.JobLaunchSpecAndWaitResponse(
            jobs=job_pb2.JobSeq(jobs=[job_pb2.Job(name=TEST_JOB_NAME)]))
        getStubMock.return_value = stubMock

        jobs = opencue.api.launchSpecAndWait(spec)

        stubMock.LaunchSpecAndWait.assert_called_with(
            job_pb2.JobLaunchSpecAndWaitRequest(spec=spec), timeout=mock.ANY)
        self.assertEqual([TEST_JOB_NAME], [job.name() for job in jobs])


class LayerTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindLayer(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.FindLayer.return_value = job_pb2.LayerFindLayerResponse(
            layer=job_pb2.Layer(name=TEST_LAYER_NAME))
        getStubMock.return_value = stubMock

        layer = opencue.api.findLayer(TEST_JOB_NAME, TEST_LAYER_NAME)

        stubMock.FindLayer.assert_called_with(
            job_pb2.LayerFindLayerRequest(job=TEST_JOB_NAME, layer=TEST_LAYER_NAME),
            timeout=mock.ANY)
        self.assertEqual(TEST_LAYER_NAME, layer.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetLayer(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.GetLayer.return_value = job_pb2.LayerGetLayerResponse(
            layer=job_pb2.Layer(id=arbitraryId))
        getStubMock.return_value = stubMock

        layer = opencue.api.getLayer(arbitraryId)

        stubMock.GetLayer.assert_called_with(
            job_pb2.LayerGetLayerRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, layer.id())


class FrameTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindFrame(self, getStubMock):
        frameNum = 4
        stubMock = mock.Mock()
        stubMock.FindFrame.return_value = job_pb2.FrameFindFrameResponse(
            frame=job_pb2.Frame(layer_name=TEST_LAYER_NAME, number=frameNum))
        getStubMock.return_value = stubMock

        frame = opencue.api.findFrame(TEST_JOB_NAME, TEST_LAYER_NAME, frameNum)

        stubMock.FindFrame.assert_called_with(
            job_pb2.FrameFindFrameRequest(job=TEST_JOB_NAME, layer=TEST_LAYER_NAME, frame=frameNum),
            timeout=mock.ANY)
        self.assertEqual(TEST_LAYER_NAME, frame.layer())
        self.assertEqual(frameNum, frame.number())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetFrame(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.GetFrame.return_value = job_pb2.FrameGetFrameResponse(
            frame=job_pb2.Frame(id=arbitraryId))
        getStubMock.return_value = stubMock

        frame = opencue.api.getFrame(arbitraryId)

        stubMock.GetFrame.assert_called_with(
            job_pb2.FrameGetFrameRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, frame.id())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetFrames(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetFrames.return_value = job_pb2.FrameGetFramesResponse(
            frames=job_pb2.FrameSeq(frames=[
                job_pb2.Frame(layer_name=TEST_LAYER_NAME, number=1),
                job_pb2.Frame(layer_name=TEST_LAYER_NAME, number=2),
                job_pb2.Frame(layer_name=TEST_LAYER_NAME, number=3),
                job_pb2.Frame(layer_name=TEST_LAYER_NAME, number=4),
                job_pb2.Frame(layer_name=TEST_LAYER_NAME, number=5),
            ]))
        getStubMock.return_value = stubMock

        frames = opencue.api.getFrames(TEST_JOB_NAME, range="1-5")
        framePageLimit = opencue.api.search.FrameSearch.limit

        stubMock.GetFrames.assert_called_with(
            job_pb2.FrameGetFramesRequest(
                job=TEST_JOB_NAME, r=job_pb2.FrameSearchCriteria(
                    frame_range="1-5", page=1, limit=framePageLimit)),
            timeout=mock.ANY)
        self.assertEqual(5, len(frames))
        self.assertTrue(all((frame.layer() == TEST_LAYER_NAME for frame in frames)))
        self.assertEqual([1, 2, 3, 4, 5], [frame.number() for frame in frames])


class ServiceTests(unittest.TestCase):
    testName = 'unittesting'
    testTags = ['playblast', 'util']
    testThreadable = False
    testMinCores = 1000
    testMaxCores = 2000
    minMemoryIncrease = 2068378

    def setUp(self):
        self.service = service_pb2.Service(
                name=self.testName, threadable=self.testThreadable, min_cores=self.testMinCores,
                max_cores=self.testMaxCores, tags=self.testTags,
                min_memory_increase=self.minMemoryIncrease)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testCreate(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.CreateService.return_value = service_pb2.ServiceCreateServiceResponse(
            service=self.service)
        getStubMock.return_value = stubMock

        newService = opencue.api.createService(self.service)

        stubMock.CreateService.assert_called_with(
            service_pb2.ServiceCreateServiceRequest(data=self.service), timeout=mock.ANY)

        self.assertEqual(self.testName, newService.name())
        self.assertEqual(self.testTags, newService.tags())
        self.assertEqual(self.testThreadable, newService.threadable())
        self.assertEqual(self.testMinCores, newService.minCores())
        self.assertEqual(self.testMaxCores, newService.maxCores())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetService.return_value = service_pb2.ServiceGetServiceResponse(
            service=self.service)
        stubMock.Delete.return_value = service_pb2.ServiceDeleteResponse()
        getStubMock.return_value = stubMock

        opencue.api.getService(self.testName).delete()

        stubMock.Delete.assert_called_with(
            service_pb2.ServiceDeleteRequest(service=self.service), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGet(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetService.return_value = service_pb2.ServiceGetServiceResponse(
            service=self.service)
        getStubMock.return_value = stubMock

        service = opencue.api.getService(self.testName)

        stubMock.GetService.assert_called_with(
            service_pb2.ServiceGetServiceRequest(name=self.testName), timeout=mock.ANY)
        self.assertEqual(self.testName, service.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testUpdate(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetService.return_value = service_pb2.ServiceGetServiceResponse(
            service=self.service)
        stubMock.Update.return_value = service_pb2.ServiceUpdateResponse()
        getStubMock.return_value = stubMock

        updatedService = opencue.api.getService(self.testName)
        updatedService.setTags(['util'])
        updatedService.update()

        stubMock.Update.assert_called_with(
            service_pb2.ServiceUpdateRequest(service=updatedService.data), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetDefault(self, getStubMock):
        stubMock = mock.Mock()
        service1 = 'service1'
        service2 = 'service2'
        stubMock.GetDefaultServices.return_value = service_pb2.ServiceGetDefaultServicesResponse(
            services=service_pb2.ServiceSeq(
                services=[service_pb2.Service(name=service1), service_pb2.Service(name=service2)]))
        getStubMock.return_value = stubMock

        services = opencue.api.getDefaultServices()

        stubMock.GetDefaultServices.assert_called_with(
            service_pb2.ServiceGetDefaultServicesRequest(), timeout=mock.ANY)
        self.assertEqual([service1, service2], [service.name() for service in services])


class SubscriptionTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindSubscription(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Find.return_value = subscription_pb2.SubscriptionFindResponse(
            subscription=subscription_pb2.Subscription(name=TEST_SUB_NAME))
        getStubMock.return_value = stubMock

        sub = opencue.api.findSubscription(TEST_SUB_NAME)

        stubMock.Find.assert_called_with(
            subscription_pb2.SubscriptionFindRequest(name=TEST_SUB_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_SUB_NAME, sub.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetSubscription(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.Get.return_value = subscription_pb2.SubscriptionGetResponse(
            subscription=subscription_pb2.Subscription(id=arbitraryId))
        getStubMock.return_value = stubMock

        sub = opencue.api.getSubscription(arbitraryId)

        stubMock.Get.assert_called_with(
            subscription_pb2.SubscriptionGetRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, sub.id())


class HostTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetHosts(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetHosts.return_value = host_pb2.HostGetHostsResponse(
            hosts=host_pb2.HostSeq(hosts=[host_pb2.Host(name=TEST_HOST_NAME)]))
        getStubMock.return_value = stubMock

        hosts = opencue.api.getHosts(name=[TEST_HOST_NAME])

        stubMock.GetHosts.assert_called_with(
            host_pb2.HostGetHostsRequest(r=host_pb2.HostSearchCriteria(hosts=[TEST_HOST_NAME])),
            timeout=mock.ANY)
        self.assertEqual(1, len(hosts))
        self.assertEqual(TEST_HOST_NAME, hosts[0].name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindHost(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.FindHost.return_value = host_pb2.HostFindHostResponse(
            host=host_pb2.Host(name=TEST_HOST_NAME))
        getStubMock.return_value = stubMock

        host = opencue.api.findHost(TEST_HOST_NAME)

        stubMock.FindHost.assert_called_with(
            host_pb2.HostFindHostRequest(name=TEST_HOST_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_HOST_NAME, host.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetHost(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.GetHost.return_value = host_pb2.HostGetHostResponse(
            host=host_pb2.Host(id=arbitraryId))
        getStubMock.return_value = stubMock

        host = opencue.api.getHost(arbitraryId)

        stubMock.GetHost.assert_called_with(
            host_pb2.HostGetHostRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, host.id())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetHostWhiteboard(self, getStubMock):
        hostId1 = 'host-one'
        hostId2 = 'host-two'
        stubMock = mock.Mock()
        stubMock.GetHostWhiteboard.return_value = host_pb2.HostGetHostWhiteboardResponse(
            nested_hosts=host_pb2.NestedHostSeq(
                nested_hosts=[host_pb2.NestedHost(id=hostId1), host_pb2.NestedHost(id=hostId2)]))
        getStubMock.return_value = stubMock

        hosts = opencue.api.getHostWhiteboard()

        stubMock.GetHostWhiteboard.assert_called_with(
            host_pb2.HostGetHostWhiteboardRequest(), timeout=mock.ANY)
        self.assertEqual([hostId1, hostId2], [host.id() for host in hosts])


class SystemTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetHost(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetSystemStats.return_value = cue_pb2.CueGetSystemStatsResponse()
        getStubMock.return_value = stubMock

        opencue.api.getSystemStats()

        stubMock.GetSystemStats.assert_called_with(
            cue_pb2.CueGetSystemStatsRequest(), timeout=mock.ANY)

class FacilityTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testCreateFacility(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Create.return_value = facility_pb2.FacilityCreateResponse(
            facility=facility_pb2.Facility(name=TEST_FACILITY_NAME))
        getStubMock.return_value = stubMock

        newFacility = opencue.api.createFacility(TEST_FACILITY_NAME)

        stubMock.Create.assert_called_with(
            facility_pb2.FacilityCreateRequest(name=TEST_FACILITY_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_FACILITY_NAME, newFacility.name)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetFacility(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Get.return_value = facility_pb2.FacilityGetResponse(
            facility=facility_pb2.Facility(name=TEST_FACILITY_NAME))
        getStubMock.return_value = stubMock

        facility = opencue.api.getFacility(TEST_FACILITY_NAME)

        stubMock.Get.assert_called_with(
            facility_pb2.FacilityGetRequest(name=TEST_FACILITY_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_FACILITY_NAME, facility.name)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testRenameFacility(self, getStubMock):
        facility = facility_pb2.Facility(name=TEST_FACILITY_NAME)
        newName = 'new-name'
        stubMock = mock.Mock()
        stubMock.Rename.return_value = facility_pb2.FacilityRenameResponse()
        getStubMock.return_value = stubMock

        opencue.api.renameFacility(facility, newName)

        stubMock.Rename.assert_called_with(
            facility_pb2.FacilityRenameRequest(facility=facility, new_name=newName),
            timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testDeleteFacility(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = facility_pb2.FacilityDeleteResponse()
        getStubMock.return_value = stubMock

        opencue.api.deleteFacility(TEST_FACILITY_NAME)

        stubMock.Delete.assert_called_with(
            facility_pb2.FacilityDeleteRequest(name=TEST_FACILITY_NAME), timeout=mock.ANY)


class DependTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetDepend(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.GetDepend.return_value = depend_pb2.DependGetDependResponse(
            depend=depend_pb2.Depend(id=arbitraryId))
        getStubMock.return_value = stubMock

        depend = opencue.api.getDepend(arbitraryId)

        stubMock.GetDepend.assert_called_with(
            depend_pb2.DependGetDependRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, depend.id())


class OwnerTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetOwner(self, getStubMock):
        ownerName = 'arbitrary-name'
        stubMock = mock.Mock()
        stubMock.GetOwner.return_value = host_pb2.OwnerGetOwnerResponse(
            owner=host_pb2.Owner(name=ownerName))
        getStubMock.return_value = stubMock

        owner = opencue.api.getOwner(ownerName)

        stubMock.GetOwner.assert_called_with(
            host_pb2.OwnerGetOwnerRequest(name=ownerName), timeout=mock.ANY)
        self.assertEqual(ownerName, owner.name())


class FilterTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindFilter(self, getStubMock):
        filterName = 'arbitrary-name'
        stubMock = mock.Mock()
        stubMock.FindFilter.return_value = filter_pb2.FilterFindFilterResponse(
            filter=filter_pb2.Filter(name=filterName))
        getStubMock.return_value = stubMock

        filterReturned = opencue.api.findFilter(TEST_SHOW_NAME, filterName)

        stubMock.FindFilter.assert_called_with(
            filter_pb2.FilterFindFilterRequest(show=TEST_SHOW_NAME, name=filterName),
            timeout=mock.ANY)
        self.assertEqual(filterName, filterReturned.name())


class AllocTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testCreateAlloc(self, getStubMock):
        facility = facility_pb2.Facility(name=TEST_FACILITY_NAME)
        stubMock = mock.Mock()
        stubMock.Create.return_value = facility_pb2.AllocCreateResponse(
            allocation=facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        getStubMock.return_value = stubMock

        alloc = opencue.api.createAllocation(TEST_ALLOC_NAME, TEST_TAG, facility)

        stubMock.Create.assert_called_with(
            facility_pb2.AllocCreateRequest(name=TEST_ALLOC_NAME, tag=TEST_TAG, facility=facility),
            timeout=mock.ANY)
        self.assertEqual(TEST_ALLOC_NAME, alloc.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetAllocs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetAll.return_value = facility_pb2.AllocGetAllResponse(
            allocations=facility_pb2.AllocationSeq(
                allocations=[facility_pb2.Allocation(name=TEST_ALLOC_NAME)]))
        getStubMock.return_value = stubMock

        allocs = opencue.api.getAllocations()

        stubMock.GetAll.assert_called_with(
            facility_pb2.AllocGetAllRequest(), timeout=mock.ANY)
        self.assertEqual([TEST_ALLOC_NAME], [alloc.name() for alloc in allocs])

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindAlloc(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Find.return_value = facility_pb2.AllocFindResponse(
            allocation=facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        getStubMock.return_value = stubMock

        alloc = opencue.api.findAllocation(TEST_ALLOC_NAME)

        stubMock.Find.assert_called_with(
            facility_pb2.AllocFindRequest(name=TEST_ALLOC_NAME), timeout=mock.ANY)
        self.assertEqual(TEST_ALLOC_NAME, alloc.name())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetAlloc(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.Get.return_value = facility_pb2.AllocGetResponse(
            allocation=facility_pb2.Allocation(id=arbitraryId))
        getStubMock.return_value = stubMock

        alloc = opencue.api.getAllocation(arbitraryId)

        stubMock.Get.assert_called_with(
            facility_pb2.AllocGetRequest(id=arbitraryId), timeout=mock.ANY)
        self.assertEqual(arbitraryId, alloc.id())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testDeleteAlloc(self, getStubMock):
        allocToDelete = facility_pb2.Allocation(name=TEST_ALLOC_NAME)
        stubMock = mock.Mock()
        stubMock.Delete.return_value = facility_pb2.AllocDeleteResponse()
        getStubMock.return_value = stubMock

        opencue.api.deleteAllocation(allocToDelete)

        stubMock.Delete.assert_called_with(
            facility_pb2.AllocDeleteRequest(allocation=allocToDelete), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetDefaultAlloc(self, getStubMock):
        arbitraryId = '00000000-0000-0000-0000-012345678980'
        stubMock = mock.Mock()
        stubMock.GetDefault.return_value = facility_pb2.AllocGetDefaultResponse(
            allocation=facility_pb2.Allocation(id=arbitraryId))
        getStubMock.return_value = stubMock

        alloc = opencue.api.getDefaultAllocation()

        stubMock.GetDefault.assert_called_with(
            facility_pb2.AllocGetDefaultRequest(), timeout=mock.ANY)
        self.assertEqual(arbitraryId, alloc.id())

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testSetDefaultAlloc(self, getStubMock):
        alloc = facility_pb2.Allocation(name=TEST_ALLOC_NAME)
        stubMock = mock.Mock()
        stubMock.SetDefault.return_value = facility_pb2.AllocSetDefaultResponse()
        getStubMock.return_value = stubMock

        opencue.api.setDefaultAllocation(alloc)

        stubMock.SetDefault.assert_called_with(
            facility_pb2.AllocSetDefaultRequest(allocation=alloc), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testAllocSetBillable(self, getStubMock):
        alloc = facility_pb2.Allocation(name=TEST_ALLOC_NAME)
        isBillable = True
        stubMock = mock.Mock()
        stubMock.SetBillable.return_value = facility_pb2.AllocSetBillableResponse()
        getStubMock.return_value = stubMock

        opencue.api.allocSetBillable(alloc, isBillable)

        stubMock.SetBillable.assert_called_with(
            facility_pb2.AllocSetBillableRequest(allocation=alloc, value=isBillable),
            timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testAllocSetName(self, getStubMock):
        alloc = facility_pb2.Allocation(name=TEST_ALLOC_NAME)
        newName = 'arbitrary-name'
        stubMock = mock.Mock()
        stubMock.SetName.return_value = facility_pb2.AllocSetNameResponse()
        getStubMock.return_value = stubMock

        opencue.api.allocSetName(alloc, newName)

        stubMock.SetName.assert_called_with(
            facility_pb2.AllocSetNameRequest(allocation=alloc, name=newName), timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testAllocSetTag(self, getStubMock):
        alloc = facility_pb2.Allocation(name=TEST_ALLOC_NAME)
        newTag = 'arbitrary-tag'
        stubMock = mock.Mock()
        stubMock.SetTag.return_value = facility_pb2.AllocSetTagResponse()
        getStubMock.return_value = stubMock

        opencue.api.allocSetTag(alloc, newTag)

        stubMock.SetTag.assert_called_with(
            facility_pb2.AllocSetTagRequest(allocation=alloc, tag=newTag), timeout=mock.ANY)


class ProcTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetProcs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetProcs.return_value = host_pb2.ProcGetProcsResponse(
            procs=host_pb2.ProcSeq(procs=[host_pb2.Proc(name=TEST_PROC_NAME)]))
        getStubMock.return_value = stubMock

        procs = opencue.api.getProcs(show=[TEST_SHOW_NAME], alloc=[TEST_ALLOC_NAME])

        stubMock.GetProcs.assert_called_with(
            host_pb2.ProcGetProcsRequest(
                r=host_pb2.ProcSearchCriteria(shows=[TEST_SHOW_NAME], allocs=[TEST_ALLOC_NAME])),
            timeout=mock.ANY)
        self.assertEqual([TEST_PROC_NAME], [proc.name() for proc in procs])


class LimitTests(unittest.TestCase):

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testCreateLimit(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Create.return_value = limit_pb2.LimitCreateResponse()
        getStubMock.return_value = stubMock

        testLimitValue = 42
        opencue.api.createLimit(TEST_LIMIT_NAME, testLimitValue)

        stubMock.Create.assert_called_with(
            limit_pb2.LimitCreateRequest(name=TEST_LIMIT_NAME, max_value=testLimitValue),
            timeout=mock.ANY)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testGetLimits(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetAll.return_value = limit_pb2.LimitGetAllResponse(
            limits=[limit_pb2.Limit(name=TEST_LIMIT_NAME)])
        getStubMock.return_value = stubMock

        limits = opencue.api.getLimits()

        stubMock.GetAll.assert_called_with(limit_pb2.LimitGetAllRequest(), timeout=mock.ANY)
        self.assertEqual(len(limits), 1)
        self.assertEqual(limits[0].name(), TEST_LIMIT_NAME)

    @mock.patch('opencue.cuebot.Cuebot.getStub')
    def testFindLimit(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Find.return_value = limit_pb2.LimitFindResponse(
            limit=limit_pb2.Limit(name=TEST_LIMIT_NAME, max_value=42))
        getStubMock.return_value = stubMock

        limit = opencue.api.findLimit(TEST_LIMIT_NAME)
        self.assertEqual(TEST_LIMIT_NAME, limit.name())
        self.assertEqual(42, limit.maxValue())

        stubMock.Find.assert_called_with(
            limit_pb2.LimitFindRequest(name=TEST_LIMIT_NAME),
            timeout=mock.ANY)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.filter`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import filter_pb2
from opencue_proto import job_pb2
import opencue.wrappers.filter
import opencue.wrappers.group
import opencue.wrappers.job


TEST_ACTION_ID = 'aaa-aaaa-aaa'
TEST_FILTER_NAME = 'testFilter'
TEST_MATCHER_ID = 'mmm-mmmm-mmm'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class FilterTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = filter_pb2.FilterDeleteResponse()
        getStubMock.return_value = stubMock

        filterToDelete = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToDelete.delete()

        stubMock.Delete.assert_called_with(
            filter_pb2.FilterDeleteRequest(filter=filterToDelete.data), timeout=mock.ANY)

    def testCreateMatcher(self, getStubMock):
        matcherId = 'mmm-mmmm-mmm'
        stubMock = mock.Mock()
        stubMock.CreateMatcher.return_value = filter_pb2.FilterCreateMatcherResponse(
            matcher=filter_pb2.Matcher(id=matcherId))
        getStubMock.return_value = stubMock

        queryStr = 'john'
        subject = filter_pb2.USER
        matcherType = filter_pb2.IS_NOT
        matcherData = opencue.wrappers.filter.MatcherData(
            subject=subject, type=matcherType, input=queryStr)
        filterForMatcher = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        matcher = filterForMatcher.createMatcher(subject, matcherType, queryStr)

        stubMock.CreateMatcher.assert_called_with(
            filter_pb2.FilterCreateMatcherRequest(filter=filterForMatcher.data, data=matcherData),
            timeout=mock.ANY)
        self.assertEqual(matcher.id(), matcherId)

    def testCreateAction(self, getStubMock):
        actionId = 'aaa-aaaa-aaa'
        stubMock = mock.Mock()
        stubMock.CreateAction.return_value = filter_pb2.FilterCreateActionResponse(
            action=filter_pb2.Action(id=actionId))
        getStubMock.return_value = stubMock

        actionType = filter_pb2.PAUSE_JOB
        value = 10
        filterForAction = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        action = filterForAction.createAction(actionType, value)
        actionData = opencue.wrappers.filter.ActionData(
            type=actionType,
            value_type=filter_pb2.INTEGER_TYPE,
            group_value=None,
            string_value=None,
            integer_value=value,
            float_value=0.0,
            boolean_value=False)

        stubMock.CreateAction.assert_called_with(
            filter_pb2.FilterCreateActionRequest(filter=filterForAction.data, data=actionData),
            timeout=mock.ANY)
        self.assertEqual(action.id(), actionId)

    def testGetActions(self, getStubMock):
        actionId = 'aaa-aaaa-aaa'
        stubMock = mock.Mock()
        stubMock.GetActions.return_value = filter_pb2.FilterGetActionsResponse(
            actions=filter_pb2.ActionSeq(actions=[filter_pb2.Action(id=actionId)]))
        getStubMock.return_value = stubMock

        filterForActions = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        actions = filterForActions.getActions()

        stubMock.GetActions.assert_called_with(
            filter_pb2.FilterGetActionsRequest(filter=filterForActions.data), timeout=mock.ANY)
        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0].id(), actionId)

    def testGetMatchers(self, getStubMock):
        matcherId = 'mmm-mmmm-mmm'
        stubMock = mock.Mock()
        stubMock.GetMatchers.return_value = filter_pb2.FilterGetMatchersResponse(
            matchers=filter_pb2.MatcherSeq(matchers=[filter_pb2.Matcher(id=matcherId)]))
        getStubMock.return_value = stubMock

        filterForMatchers = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        matchers = filterForMatchers.getMatchers()

        stubMock.GetMatchers.assert_called_with(
            filter_pb2.FilterGetMatchersRequest(filter=filterForMatchers.data), timeout=mock.ANY)
        self.assertEqual(len(matchers), 1)
        self.assertEqual(matchers[0].id(), matcherId)

    def testLowerOrder(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.LowerOrder.return_value = filter_pb2.FilterLowerOrderResponse()
        getStubMock.return_value = stubMock

        filterInst = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterInst.lowerOrder()

        stubMock.LowerOrder.assert_called_with(
            filter_pb2.FilterLowerOrderRequest(filter=filterInst.data), timeout=mock.ANY)

    def testRaiseOrder(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RaiseOrder.return_value = filter_pb2.FilterRaiseOrderResponse()
        getStubMock.return_value = stubMock

        filterInst = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterInst.raiseOrder()

        stubMock.RaiseOrder.assert_called_with(
            filter_pb2.FilterRaiseOrderRequest(filter=filterInst.data), timeout=mock.ANY)

    def testOrderFirst(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.OrderFirst.return_value = filter_pb2.FilterOrderFirstResponse()
        getStubMock.return_value = stubMock

        filterInst = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterInst.orderFirst()

        stubMock.OrderFirst.assert_called_with(
            filter_pb2.FilterOrderFirstRequest(filter=filterInst.data), timeout=mock.ANY)

    def testOrderLast(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.OrderLast.return_value = filter_pb2.FilterOrderLastResponse()
        getStubMock.return_value = stubMock

        filterInst = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterInst.orderLast()

        stubMock.OrderLast.assert_called_with(
            filter_pb2.FilterOrderLastRequest(filter=filterInst.data), timeout=mock.ANY)

    def testRunFilterOnGroup(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RunFilterOnGroup.return_value = filter_pb2.FilterRunFilterOnGroupResponse()
        getStubMock.return_value = stubMock

        group = opencue.wrappers.group.Group(job_pb2.Group(name='testGroup'))
        filterToRun = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToRun.runFilterOnGroup(group)

        stubMock.RunFilterOnGroup.assert_called_with(
            filter_pb2.FilterRunFilterOnGroupRequest(filter=filterToRun.data, group=group.data),
            timeout=mock.ANY)

    def testRunFilterOnJobs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RunFilterOnJobs.return_value = filter_pb2.FilterRunFilterOnJobsResponse()
        getStubMock.return_value = stubMock

        jobs = [opencue.wrappers.job.Job(job_pb2.Job(name='testJob'))]
        jobSeq = job_pb2.JobSeq(jobs=[job.data for job in jobs])
        filterToRun = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToRun.runFilterOnJobs(jobs)

        stubMock.RunFilterOnJobs.assert_called_with(
            filter_pb2.FilterRunFilterOnJobsRequest(filter=filterToRun.data, jobs=jobSeq),
            timeout=mock.ANY)

    def testSetEnabled(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetEnabled.return_value = filter_pb2.FilterSetEnabledResponse()
        getStubMock.return_value = stubMock

        value = True
        filterToEnable = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToEnable.setEnabled(value)

        stubMock.SetEnabled.assert_called_with(
            filter_pb2.FilterSetEnabledRequest(
                filter=filterToEnable.data, enabled=value), timeout=mock.ANY)

    def testSetName(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetName.return_value = filter_pb2.FilterSetNameResponse()
        getStubMock.return_value = stubMock

        value = 'newname'
        filterToSet = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToSet.setName(value)

        stubMock.SetName.assert_called_with(
            filter_pb2.FilterSetNameRequest(filter=filterToSet.data, name=value), timeout=mock.ANY)

    def testSetType(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetType.return_value = filter_pb2.FilterSetTypeResponse()
        getStubMock.return_value = stubMock

        value = filter_pb2.MATCH_ALL
        filterToSet = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToSet.setType(value)

        stubMock.SetType.assert_called_with(
            filter_pb2.FilterSetTypeRequest(filter=filterToSet.data, type=value), timeout=mock.ANY)

    def testSetOrder(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetOrder.return_value = filter_pb2.FilterSetOrderResponse()
        getStubMock.return_value = stubMock

        value = 2
        filterToSet = opencue.wrappers.filter.Filter(filter_pb2.Filter(name=TEST_FILTER_NAME))
        filterToSet.setOrder(value)

        stubMock.SetOrder.assert_called_with(
            filter_pb2.FilterSetOrderRequest(
                filter=filterToSet.data, order=value), timeout=mock.ANY)


@mock.patch('opencue.cuebot.Cuebot.getStub')
class ActionTests(unittest.TestCase):

    def testGetParentFilter(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetParentFilter.return_value = filter_pb2.ActionGetParentFilterResponse(
            filter=filter_pb2.Filter(name=TEST_FILTER_NAME))
        getStubMock.return_value = stubMock

        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))
        filterReturned = action.getParentFilter()

        stubMock.GetParentFilter.assert_called_with(
            filter_pb2.ActionGetParentFilterRequest(action=action.data), timeout=mock.ANY)
        self.assertEqual(filterReturned.name(), TEST_FILTER_NAME)

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = filter_pb2.ActionDeleteResponse()
        getStubMock.return_value = stubMock

        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))
        action.delete()

        stubMock.Delete.assert_called_with(
            filter_pb2.ActionDeleteRequest(action=action.data), timeout=mock.ANY)

    def testCommitNew(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        action = opencue.wrappers.filter.Action()
        with self.assertRaises(Exception):
            action.commit()

    def testCommit(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))
        action.commit()

        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)

    def testIsNew(self, getStubMock):
        actionFalse = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))
        self.assertFalse(actionFalse.isNew())

        actionTrue = opencue.wrappers.filter.Action()
        self.assertTrue(actionTrue.isNew())

    def testName(self, getStubMock):
        actionInt = opencue.wrappers.filter.Action(filter_pb2.Action(
            id=TEST_ACTION_ID, type=filter_pb2.PAUSE_JOB, value_type=filter_pb2.INTEGER_TYPE,
            integer_value=22))
        self.assertEqual(actionInt.name(), "PAUSE_JOB 22")

        actionNone = opencue.wrappers.filter.Action(filter_pb2.Action(
            id=TEST_ACTION_ID, type=filter_pb2.PAUSE_JOB, value_type=filter_pb2.NONE_TYPE))
        self.assertEqual(actionNone.name(), "PAUSE_JOB")

    def testValue(self, getStubMock):
        groupValue = 'testGroup'
        stringValue = 'testString'
        intValue = 22
        floatValue = 22.2
        boolValue = True
        action = opencue.wrappers.filter.Action(
            filter_pb2.Action(
                id=TEST_ACTION_ID,
                type=filter_pb2.PAUSE_JOB,
                group_value=groupValue,
                string_value=stringValue,
                integer_value=intValue,
                float_value=floatValue,
                boolean_value=boolValue))

        action.data.value_type = filter_pb2.GROUP_TYPE
        self.assertEqual(action.value(), groupValue)

        action.data.value_type = filter_pb2.STRING_TYPE
        self.assertEqual(action.value(), stringValue)

        action.data.value_type = filter_pb2.INTEGER_TYPE
        self.assertEqual(action.value(), intValue)

        action.data.value_type = filter_pb2.FLOAT_TYPE
        threshold = abs(action.value() - floatValue)
        self.assertTrue(threshold < 0.0001)

        action.data.value_type = filter_pb2.BOOLEAN_TYPE
        self.assertEqual(action.value(), boolValue)

        action.data.value_type = filter_pb2.NONE_TYPE
        self.assertEqual(action.value(), None)

    def testSetTypeAndValueFail(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        with self.assertRaises(Exception):
            action.setTypeAndValue('foo', 'bar')

    def testSetTypeAndValueGroup(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        testData = (filter_pb2.MOVE_JOB_TO_GROUP, job_pb2.Group(id='testGroup'))
        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        action.setTypeAndValue(testData[0], testData[1])
        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)

    def testSetTypeAndValueString(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        testData = (filter_pb2.SET_ALL_RENDER_LAYER_TAGS, 'testString')
        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        action.setTypeAndValue(testData[0], testData[1])
        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)

    def testSetTypeAndValueInt(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        testData = (filter_pb2.SET_JOB_PRIORITY, 22)
        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        action.setTypeAndValue(testData[0], testData[1])
        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)

    def testSetTypeAndValueFloat(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        testData = (filter_pb2.SET_JOB_MIN_CORES, 22.2)
        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        action.setTypeAndValue(testData[0], testData[1])
        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)

    def testSetTypeAndValueBool(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        testData = (filter_pb2.PAUSE_JOB, True)
        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        action.setTypeAndValue(testData[0], testData[1])
        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)

    def testSetTypeAndValueNone(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.ActionCommitResponse()
        getStubMock.return_value = stubMock

        testData = (filter_pb2.STOP_PROCESSING, None)
        action = opencue.wrappers.filter.Action(filter_pb2.Action(id=TEST_ACTION_ID))

        action.setTypeAndValue(testData[0], testData[1])
        stubMock.Commit.assert_called_with(
            filter_pb2.ActionCommitRequest(action=action.data), timeout=mock.ANY)


@mock.patch('opencue.cuebot.Cuebot.getStub')
class MatcherTests(unittest.TestCase):

    def testGetParentFilter(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetParentFilter.return_value = filter_pb2.MatcherGetParentFilterResponse(
            filter=filter_pb2.Filter(name=TEST_FILTER_NAME))
        getStubMock.return_value = stubMock

        matcher = opencue.wrappers.filter.Matcher(filter_pb2.Matcher(id=TEST_MATCHER_ID))
        filterReturns = matcher.getParentFilter()

        stubMock.GetParentFilter.assert_called_with(
            filter_pb2.MatcherGetParentFilterRequest(matcher=matcher.data), timeout=mock.ANY)
        self.assertEqual(filterReturns.name(), TEST_FILTER_NAME)

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = filter_pb2.MatcherDeleteResponse()
        getStubMock.return_value = stubMock

        matcher = opencue.wrappers.filter.Matcher(
            filter_pb2.Matcher(id=TEST_MATCHER_ID))
        matcher.delete()

        stubMock.Delete.assert_called_with(
            filter_pb2.MatcherDeleteRequest(matcher=matcher.data), timeout=mock.ANY)

    def testCommitNew(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = filter_pb2.MatcherDeleteResponse()
        getStubMock.return_value = stubMock

        matcher = opencue.wrappers.filter.Matcher()
        with self.assertRaises(Exception):
            matcher.commit()

    def testCommit(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Commit.return_value = filter_pb2.MatcherCommitResponse()
        getStubMock.return_value = stubMock

        matcher = opencue.wrappers.filter.Matcher(
            filter_pb2.Matcher(id=TEST_MATCHER_ID, input='test'))
        matcher.commit()

        stubMock.Commit.assert_called_with(
            filter_pb2.MatcherCommitRequest(matcher=matcher.data), timeout=mock.ANY)

    def testIsNew(self, getStubMock):
        matcherFalse = opencue.wrappers.filter.Matcher(
            filter_pb2.Matcher(id=TEST_MATCHER_ID, input='test'))
        self.assertFalse(matcherFalse.isNew())

        matcherTrue = opencue.wrappers.filter.Matcher()
        self.assertTrue(matcherTrue.isNew())


class FilterEnumTests(unittest.TestCase):

    def testFilterType(self):
        self.assertEqual(opencue.api.Filter.FilterType.MATCH_ANY,
                         filter_pb2.MATCH_ANY)
        self.assertEqual(opencue.api.Filter.FilterType.MATCH_ANY, 0)


class ActionEnumTests(unittest.TestCase):

    def testActionType(self):
        self.assertEqual(opencue.api.Action.ActionType.MOVE_JOB_TO_GROUP,
                         filter_pb2.MOVE_JOB_TO_GROUP)
        self.assertEqual(opencue.api.Action.ActionType.MOVE_JOB_TO_GROUP, 0)

    def testActionValueType(self):
        self.assertEqual(opencue.api.Action.ActionValueType.INTEGER_TYPE,
                         filter_pb2.INTEGER_TYPE)
        self.assertEqual(opencue.api.Action.ActionValueType.INTEGER_TYPE, 2)


class MatcherEnumTests(unittest.TestCase):

    def testMatchSubject(self):
        self.assertEqual(opencue.api.Matcher.MatchSubject.JOB_NAME,
                         filter_pb2.JOB_NAME)
        self.assertEqual(opencue.api.Matcher.MatchSubject.JOB_NAME, 0)

    def testMatchType(self):
        self.assertEqual(opencue.api.Matcher.MatchType.IS,
                         filter_pb2.IS)
        self.assertEqual(opencue.api.Matcher.MatchType.IS, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.host`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import os
import unittest

import mock

from opencue_proto import comment_pb2
from opencue_proto import facility_pb2
from opencue_proto import host_pb2
from opencue_proto import renderPartition_pb2
import opencue.wrappers.allocation
import opencue.wrappers.host


TEST_HOST_NAME = 'testHost'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class HostTests(unittest.TestCase):

    def testLock(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Lock.return_value = host_pb2.HostLockResponse()
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.lock()

        stubMock.Lock.assert_called_with(
            host_pb2.HostLockRequest(host=host.data),
            timeout=mock.ANY)

    def testUnlock(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Unlock.return_value = host_pb2.HostUnlockResponse()
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.unlock()

        stubMock.Unlock.assert_called_with(
            host_pb2.HostUnlockRequest(host=host.data),
            timeout=mock.ANY)

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = host_pb2.HostDeleteResponse()
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.delete()

        stubMock.Delete.assert_called_with(
            host_pb2.HostDeleteRequest(host=host.data),
            timeout=mock.ANY)

    def testGetProcs(self, getStubMock):
        procName = 'testProc'
        stubMock = mock.Mock()
        stubMock.GetProcs.return_value = host_pb2.HostGetProcsResponse(
            procs=host_pb2.ProcSeq(procs=[host_pb2.Proc(name=procName)]))
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        procs = host.getProcs()

        stubMock.GetProcs.assert_called_with(
            host_pb2.HostGetProcsRequest(host=host.data),
            timeout=mock.ANY)
        self.assertEqual(len(procs), 1)
        self.assertEqual(procs[0].name(), procName)

    def testGetRenderPartitions(self, getStubMock):
        renderPartId = 'rpr-rprp-rpr'
        stubMock = mock.Mock()
        stubMock.GetRenderPartitions.return_value = host_pb2.HostGetRenderPartitionsResponse(
            render_partitions=renderPartition_pb2.RenderPartitionSeq(
                render_partitions=[renderPartition_pb2.RenderPartition(id=renderPartId)]))
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        renderParts = host.getRenderPartitions()

        stubMock.GetRenderPartitions.assert_called_with(
            host_pb2.HostGetRenderPartitionsRequest(host=host.data),
            timeout=mock.ANY)
        self.assertEqual(len(renderParts), 1)
        self.assertEqual(renderParts[0].data.id, renderPartId)

    def testRebootWhenIdle(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RebootWhenIdle.return_value = host_pb2.HostRebootWhenIdleResponse()
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.rebootWhenIdle()

        stubMock.RebootWhenIdle.assert_called_with(
            host_pb2.HostRebootWhenIdleRequest(host=host.data),
            timeout=mock.ANY)

    def testReboot(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Reboot.return_value = host_pb2.HostRebootResponse()
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.reboot()

        stubMock.Reboot.assert_called_with(
            host_pb2.HostRebootRequest(host=host.data),
            timeout=mock.ANY)

    def testAddTags(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.AddTags.return_value = host_pb2.HostAddTagsResponse()
        getStubMock.return_value = stubMock

        tags = ['tags', 'are', 'fun']
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.addTags(tags)

        stubMock.AddTags.assert_called_with(
            host_pb2.HostAddTagsRequest(host=host.data, tags=tags),
            timeout=mock.ANY)

    def testRemoveTags(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RemoveTags.return_value = host_pb2.HostRemoveTagsResponse()
        getStubMock.return_value = stubMock

        tags = ['tags', 'are', 'fun']
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.removeTags(tags)

        stubMock.RemoveTags.assert_called_with(
            host_pb2.HostRemoveTagsRequest(host=host.data, tags=tags),
            timeout=mock.ANY)

    def testRenameTag(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RenameTag.return_value = host_pb2.HostRenameTagResponse()
        getStubMock.return_value = stubMock

        oldTag = 'sad'
        newTag = 'happy'
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.renameTag(oldTag, newTag)

        stubMock.RenameTag.assert_called_with(
            host_pb2.HostRenameTagRequest(host=host.data, old_tag=oldTag, new_tag=newTag),
            timeout=mock.ANY)

    def testSetAllocation(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetAllocation.return_value = host_pb2.HostSetAllocationResponse()
        getStubMock.return_value = stubMock

        allocId = 'aaa-aaaa-aaa'
        alloc = opencue.wrappers.allocation.Allocation(facility_pb2.Allocation(id=allocId))
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.setAllocation(alloc)

        stubMock.SetAllocation.assert_called_with(
            host_pb2.HostSetAllocationRequest(host=host.data, allocation_id=alloc.id()),
            timeout=mock.ANY)

    def testAddComment(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.AddComment.return_value = host_pb2.HostAddCommentResponse()
        getStubMock.return_value = stubMock

        subject = 'test'
        message = 'this is a test.'
        comment = comment_pb2.Comment(user=os.getenv("USER", "unknown"),
                                      subject=subject,
                                      message=message,
                                      timestamp=0)
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.addComment(subject, message)

        stubMock.AddComment.assert_called_with(
            host_pb2.HostAddCommentRequest(host=host.data, new_comment=comment),
            timeout=mock.ANY)

    def testGetComments(self, getStubMock):
        message = 'this is a test.'
        stubMock = mock.Mock()
        stubMock.GetComments.return_value = host_pb2.HostGetCommentsResponse(
            comments=comment_pb2.CommentSeq(comments=[comment_pb2.Comment(message=message)]))
        getStubMock.return_value = stubMock

        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        comments = host.getComments()

        stubMock.GetComments.assert_called_with(
            host_pb2.HostGetCommentsRequest(host=host.data),
            timeout=mock.ANY)
        self.assertEqual(len(comments), 1)
        self.assertEqual(comments[0].message(), message)

    def testSetHardwareState(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetHardwareState.return_value = host_pb2.HostSetHardwareStateResponse()
        getStubMock.return_value = stubMock

        state = host_pb2.REBOOTING
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.setHardwareState(state)

        stubMock.SetHardwareState.assert_called_with(
            host_pb2.HostSetHardwareStateRequest(host=host.data, state=state),
            timeout=mock.ANY)

    def testSetOs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetOs.return_value = host_pb2.HostSetOsResponse()
        getStubMock.return_value = stubMock

        osName = 'linux'
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.setOs(osName)

        stubMock.SetOs.assert_called_with(
            host_pb2.HostSetOsRequest(host=host.data, os=osName),
            timeout=mock.ANY)

    def testSetThreadMode(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetThreadMode.return_value = host_pb2.HostSetThreadModeResponse()
        getStubMock.return_value = stubMock

        mode = host_pb2.VARIABLE
        host = opencue.wrappers.host.Host(
            host_pb2.Host(name=TEST_HOST_NAME))
        host.setThreadMode(mode)

        stubMock.SetThreadMode.assert_called_with(
            host_pb2.HostSetThreadModeRequest(host=host.data, mode=mode),
            timeout=mock.ANY)


class HostEnumTests(unittest.TestCase):

    def testHardwareState(self):
        self.assertEqual(opencue.api.Host.HardwareState.UP, host_pb2.UP)
        self.assertEqual(opencue.api.Host.HardwareState.UP, 0)

    def testHostTagType(self):
        self.assertEqual(opencue.api.Host.HostTagType.HARDWARE,
                         host_pb2.HARDWARE)
        self.assertEqual(opencue.api.Host.HostTagType.HARDWARE, 1)

    def testLockState(self):
        self.assertEqual(opencue.api.Host.LockState.NIMBY_LOCKED,
                         host_pb2.NIMBY_LOCKED)
        self.assertEqual(opencue.api.Host.LockState.NIMBY_LOCKED, 2)

    def testThreadMode(self):
        self.assertEqual(opencue.api.Host.ThreadMode.ALL,
                         host_pb2.ALL)
        self.assertEqual(opencue.api.Host.ThreadMode.ALL, 1)


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Client side implementation of search criteria.

This module provides some easy factory methods to do common search operations. It also exposes
lower level RPC functionality for procedural searches.

==============
Usage examples
==============

The following example illustrates searching using the high level API::

    jobs = getJobs(show=["pipe"])

An example of a procedural search::

    s = JobSearch()
    s.shows.append("pipe")
    s.users.append("chambers")
    jobs = s.find()

A procedural example searching by regular expression::

    s = JobSearch()
    s.includeFinished = True
    s.regex.append("blah")
    for job in s.find():
        print job

Another procedural example::

    for job in JobSearch.byUser(["chambers","jwelborn"]):
        job.proxy.kill()

"""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import division
from builtins import object
import logging

# pylint: disable=cyclic-import
from opencue_proto import criterion_pb2
from opencue_proto import host_pb2
from opencue_proto import job_pb2
import opencue.wrappers.host
from .cuebot import Cuebot

logger = logging.getLogger("opencue")

__all__ = ["BaseSearch",
           "ProcSearch",
           "FrameSearch",
           "HostSearch",
           "JobSearch"]


class BaseSearch(object):
    """Base class for searching."""

    def __init__(self, **options):
        self.options = options

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        return self.options == other.options

    def search(self):
        """Executes the search using the options provided at initiation."""
        return self.byOptions(**self.options)

    @classmethod
    def byOptions(cls, **options):
        """Executes the search using the provided options."""
        raise NotImplementedError


class ProcSearch(BaseSearch):
    """Class for searching for procs.

    See: help(opencue.getProcs)"""

    @staticmethod
    def criteriaFromOptions(**options):
        """Constructs a search criteria object for the given options."""
        return _setOptions(host_pb2.ProcSearchCriteria(), options)

    @classmethod
    def byOptions(cls, **options):
        """Executes the search using the given options."""
        criteria = cls.criteriaFromOptions(**options)
        return Cuebot.getStub('proc').GetProcs(
            host_pb2.ProcGetProcsRequest(r=criteria), timeout=Cuebot.Timeout)


class FrameSearch(BaseSearch):
    """Class for searching for frames."""

    page = 1
    limit = 500
    change_date = 0

    @classmethod
    def criteriaFromOptions(cls, **options):
        """Constructs a search criteria object for the given options."""
        criteria = _setOptions(job_pb2.FrameSearchCriteria(), options)
        criteria.page = options.get('page', cls.page)
        criteria.limit = options.get('limit', cls.limit)
        criteria.change_date = options.get('change_date', cls.change_date)
        return criteria

    # pylint: disable=arguments-differ
    @classmethod
    def byOptions(cls, job, **options):
        criteria = cls.criteriaFromOptions(**options)
        return Cuebot.getStub('frame').GetFrames(job_pb2.FrameGetFramesRequest(job=job, r=criteria),
                                                 timeout=Cuebot.Timeout)

    @classmethod
    def byRange(cls, job, val):
        """Executes a search by frame range."""
        cls.byOptions(job, frame_range=val)


class HostSearch(BaseSearch):
    """Class for searching for hosts."""

    @staticmethod
    def criteriaFromOptions(**options):
        """Constructs a search criteria object for the given options."""
        return _setOptions(host_pb2.HostSearchCriteria(), options)

    @classmethod
    def byOptions(cls, **options):
        criteria = cls.criteriaFromOptions(**options)
        return [
            opencue.wrappers.host.Host(host) for host in Cuebot.getStub('host').GetHosts(
                host_pb2.HostGetHostsRequest(r=criteria), timeout=Cuebot.Timeout).hosts.hosts]

    @classmethod
    def byName(cls, val):
        """Searches for a host by name."""
        return cls.byOptions(name=val)

    @classmethod
    def byRegex(cls, val):
        """Searches for a host by regular expression."""
        return cls.byOptions(regex=val)

    @classmethod
    def byId(cls, val):
        """Searches for a host by id."""
        return cls.byOptions(id=val)

    @classmethod
    def byMatch(cls, val):
        """Searches for a host by substring match."""
        return cls.byOptions(substr=val)

    @classmethod
    def byAllocation(cls, val):
        """Searches for a host by allocation."""
        return cls.byOptions(alloc=val)


class JobSearch(BaseSearch):
    """Class for searching for jobs."""

    @staticmethod
    def criteriaFromOptions(**options):
        """Constructs a search criteria object for the given options."""
        return _setOptions(job_pb2.JobSearchCriteria(), options)

    @classmethod
    def byOptions(cls, **options):
        criteria = cls.criteriaFromOptions(**options)
        return Cuebot.getStub('job').GetJobs(
            job_pb2.JobGetJobsRequest(r=criteria), timeout=Cuebot.Timeout)

    @classmethod
    def byName(cls, val):
        """Searches for a job by name."""
        return cls.byOptions(job=val)

    @classmethod
    def byId(cls, val):
        """Searches for a job by id."""
        return cls.byOptions(id=val)

    @classmethod
    def byRegex(cls, val):
        """Searches for a job by regex."""
        return cls.byOptions(regex=val)

    @classmethod
    def byMatch(cls, val):
        """Searches for a job by substring match."""
        return cls.byOptions(substr=val)

    @classmethod
    def byShow(cls, val):
        """Searches for a job by show."""
        return cls.byOptions(show=val)

    @classmethod
    def byShot(cls, val):
        """Searches for a job by shot."""
        return cls.byOptions(shots=val)

    @classmethod
    def byUser(cls, val):
        """Searches for a job by user."""
        return cls.byOptions(user=val)


def _append(stuff, item):
    if isinstance(item, (tuple, list, set)):
        stuff.extend(item)
    else:
        stuff.append(item)


def _createCriterion(search, searchType, convert=None):
    """handleCriterion
        returns the proper subclass of FloatSearchCriterion or IntSearchCriterion
        based on input from the user. There are a few formats which are accepted.

        float/int - GreaterThan[searchType]SearchCriterion
        string -
            gt<value> - GreaterThan[searchType]SearchCriterion
            lt<value> - LessThan[searchType]SearchCriterion
            min-max  - InRange[searchType]SearchCriterion

    :type  search: String or Int or Float
    :param search: The search desired: 'gt#', 'lt#', '#-#'.
                   '#' or # is assumed greater than.
    :type  searchType: Int or Float
    :param searchType: The type of search criterion required
    :type  convert: callable
    :param convert: Optional callable to convert the input to the units the
                    cuebot uses. ie: hours to seconds.
    :rtype:  SearchCriterion
    :return: A SearchCriterion object"""
    def _convert(val):
        if not convert:
            return searchType(val)
        return searchType(convert(searchType(val)))

    if isinstance(search, (int, float)) or \
            isinstance(search, str) and search.isdigit():
        search = "gt%s" % search

    if searchType == float:
        searchTypeStr = "Float"
    elif searchType == int:
        searchTypeStr = "Integer"
    else:
        raise ValueError("Unknown searchType, must be Int or Float")

    if search.startswith("gt"):
        criterion = getattr(criterion_pb2,
                            "GreaterThan%sSearchCriterion" % searchTypeStr)
        return criterion(value=_convert(search[2:]))

    if search.startswith("lt"):
        criterion = getattr(criterion_pb2,
                            "LessThan%sSearchCriterion" % searchTypeStr)
        return criterion(value=_convert(int(search[2:])))

    if search.find("-") > -1:
        criterion = getattr(criterion_pb2,
                            "InRange%sSearchCriterion" % searchTypeStr)
        min_range, max_range = search.split("-")
        return criterion(min=_convert(min_range), max=_convert(max_range))

    raise ValueError("Unable to parse this format: %s" % search)


def _raiseIfNotType(searchOption, value, expectedType):
    if not isinstance(value, list):
        raise TypeError("Failed to set search option: '{}'. Expects type '{}', but got {}.".format(
            searchOption, expectedType, type(value)))


def raiseIfNotList(searchOption, value):
    """Raises an exception if the provided value is not a list."""
    _raiseIfNotType(searchOption, value, list)


def _setOptions(criteria, options):

    for k, v in options.items():
        if k == "job" or (k == "name" and isinstance(criteria, job_pb2.JobSearchCriteria)):
            raiseIfNotList(k, v)
            criteria.jobs.extend(v)
        elif k == "host" or (k == "name" and isinstance(criteria, host_pb2.HostSearchCriteria)):
            raiseIfNotList(k, v)
            criteria.hosts.extend(v)
        elif k == "frames" or (k == "name" and isinstance(criteria, job_pb2.FrameSearchCriteria)):
            raiseIfNotList(k, v)
            criteria.frames.extend(v)
        elif k in("match", "substr"):
            raiseIfNotList(k, v)
            criteria.substr.extend(v)
        elif k == "regex":
            raiseIfNotList(k, v)
            criteria.regex.extend(v)
        elif k == "id":
            raiseIfNotList(k, v)
            criteria.ids.extend(v)
        elif k == "show":
            raiseIfNotList(k, v)
            criteria.shows.extend(v)
        elif k == "shot":
            raiseIfNotList(k, v)
            criteria.shots.extend(v)
        elif k == "user":
            raiseIfNotList(k, v)
            criteria.users.extend(v)
        elif k == "state" and isinstance(criteria, job_pb2.FrameSearchCriteria):
            raiseIfNotList(k, v)
            criteria.states.frame_states.extend(v)
        elif k == "state" and isinstance(criteria, host_pb2.HostSearchCriteria):
            raiseIfNotList(k, v)
            criteria.states.state.extend(v)
        elif k == "lock_state" and isinstance(criteria, host_pb2.HostSearchCriteria):
            raiseIfNotList(k, v)
            criteria.lock_states.state.extend(v)
        elif k == "layer":
            raiseIfNotList(k, v)
            criteria.layers.extend(v)
        elif k == "alloc":
            raiseIfNotList(k, v)
            criteria.allocs.extend(v)
        elif k in ("range", "frames"):
            if not v:
                continue
            if isinstance(criteria.frame_range, str):
                # Once FrameSearch.frameRange is not a string
                # this can go away
                criteria.frame_range = v
            else:
                criteria.frame_range.append(_createCriterion(v, int))
        elif k == "memory":
            if not v:
                continue
            if isinstance(criteria.memory_range, str):
                # Once FrameSearch.memoryRange is not a string
                # this can go away
                criteria.memory_range = v
            else:
                criteria.memory_range.append(
                    _createCriterion(v, int, lambda mem: (mem)))
        elif k == "memory_greater_than":
            if not v:
                continue
            criteria.memory_greater_than.append(
                    _createCriterion(v, int, lambda mem: (mem)))
        elif k == "memory_less_than":
            if not v:
                continue
            criteria.memory_greater_than.append(
                _createCriterion(v, int, lambda mem: (mem)))
        elif k == "duration":
            if not v:
                continue
            if isinstance(criteria.duration_range, str):
                # Once FrameSearch.durationRange is not a string
                # this can go away
                criteria.duration_range = v
            else:
                criteria.duration_range.append(
                    _createCriterion(v, int, lambda duration: (60 * 60 * duration)))
        elif k == "limit":
            criteria.max_results = int(v)
        elif k == "page" and isinstance(criteria, job_pb2.FrameSearchCriteria):
            criteria.page = int(v)
        elif k == "offset":
            criteria.first_result = int(v)
        elif k == "include_finished":
            criteria.include_finished = v
        elif k in ("os_filter",):
            # Client-side only options - these don't get sent to the server
            pass
        elif len(k) == 0:
            return criteria
        else:
            raise Exception("Criteria for search does not exist")
    return criteria



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for `FileSequence.FrameSet`."""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

from builtins import str
from builtins import object
from .FrameRange import FrameRange


class FrameSet(object):
    """Represents a sequence of `FileSequence.FrameRange`."""

    def __init__(self, frameRange):
        """Construct a FrameSet object by parsing a spec.

        See FrameRange for the supported syntax. A FrameSet follows the same syntax,
        with the addition that it may be a comma-separated list of different FrameRanges.
        """
        self.frameList = self.parseFrameRange(frameRange)

    def __str__(self):
        # TODO(bcipriano) Make this smarter, group frame ranges and by step. (Issue #83)
        return ','.join([str(frame) for frame in self.frameList])

    def __getitem__(self, key):
        return self.frameList[key]

    def __len__(self):
        return self.size()

    def size(self):
        """Gets the number of frames contained in this sequence."""
        return len(self.frameList)

    def get(self, idx):
        """Gets an individual entry in the sequence, by numerical position."""
        return self.frameList[idx]

    def index(self, idx):
        """Query index of frame number in frame set.

        Returns:
            int, index of frame. -1 if frame set does not contain frame.
        """
        try:
            return self.frameList.index(idx)
        except ValueError:
            return -1

    def getAll(self):
        """Gets the full numerical sequence."""
        return self.frameList

    def normalize(self):
        """Sorts and dedeuplicates the sequence."""
        self.frameList = list(set(self.frameList))
        self.frameList.sort()

    @staticmethod
    def parseFrameRange(frameRange):
        """
        Parses a string representation of a frame range into a FrameSet.

        :type frameRange: str
        :param frameRange: String representation of the frame range.
        :rtype: FrameSet
        :return: The FrameSet representing the same sequence.
        """
        frameList = []
        for frameRangeSection in frameRange.split(','):
            frameList.extend(FrameRange.parseFrameRange(frameRangeSection))
        return frameList



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Classes for working with filters."""

import enum

from opencue_proto import filter_pb2
from opencue_proto import job_pb2
from opencue_proto.filter_pb2 import Action as ActionData
from opencue_proto.filter_pb2 import ActionType
from opencue_proto.filter_pb2 import ActionValueType
from opencue_proto.filter_pb2 import Filter as FilterData
from opencue_proto.filter_pb2 import FilterType
from opencue_proto.filter_pb2 import MatchSubject
from opencue_proto.filter_pb2 import MatchType
from opencue_proto.filter_pb2 import Matcher as MatcherData
from opencue import Cuebot
import opencue.wrappers.group


__all__ = ["Filter", "Action", "Matcher",
           "FilterData", "ActionData", "MatcherData",
           "FilterType", "ActionType", "MatchType",
           "ActionValueType", "MatchSubject"]


class Filter(object):
    """This class contains the grpc implementation related to a Filter."""

    class FilterType(enum.IntEnum):
        """The type of match used to determine if objects pass the filter."""
        MATCH_ANY = filter_pb2.MATCH_ANY
        MATCH_ALL = filter_pb2.MATCH_ALL

    # pylint: disable=redefined-builtin
    def __init__(self, filter=None):
        self.data = filter
        self.stub = Cuebot.getStub('filter')

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        return self.data == other.data

    def delete(self):
        """Deletes the filter."""
        self.stub.Delete(filter_pb2.FilterDeleteRequest(filter=self.data), timeout=Cuebot.Timeout)

    def createMatcher(self, subject, matchType, query):
        """Creates a matcher for the filter.

        :type  subject: filter_pb2.MatchSubject.*
        :param subject: the job attribute to match
        :type  matchType: filter_pb2.MatchType.*
        :param matchType: the type of match to perform
        :type  query: string
        :param query: the value to match
        :rtype:  Matcher
        :return: the new matcher object
        """
        matcher = MatcherData(
            subject=subject,
            type=matchType,
            input=query.replace(' ', '')
        )
        return Matcher(self.stub.CreateMatcher(
            filter_pb2.FilterCreateMatcherRequest(filter=self.data, data=matcher),
            timeout=Cuebot.Timeout).matcher)

    def createAction(self, actionType, value):
        """Creates an action for the filter.

        :type  actionType: filter_pb2.ActionType.*
        :param actionType: the action to perform
        :type  value: opencue.wrapper.group.Group / str / int / bool
        :param value: value relevant to the type selected
        :rtype:  opencue.wrappers.filter.Action
        :return: the new action
        """
        action = ActionData(
            type=actionType,
            group_value=None,
            string_value=None,
            integer_value=0,
            float_value=0.0,
            boolean_value=False
        )

        if isinstance(value, opencue.wrappers.group.Group):
            action.value_type = filter_pb2.GROUP_TYPE
            action.group_value = value.id()
        elif isinstance(value, str):
            action.value_type = filter_pb2.STRING_TYPE
            action.string_value = value
        elif isinstance(value, bool):
            action.value_type = filter_pb2.BOOLEAN_TYPE
            action.boolean_value = value
        elif isinstance(value, int):
            action.value_type = filter_pb2.INTEGER_TYPE
            action.integer_value = value
        elif isinstance(value, float):
            action.value_type = filter_pb2.FLOAT_TYPE
            action.float_value = value
        else:
            action.value_type = filter_pb2.NONE_TYPE

        return Action(self.stub.CreateAction(
            filter_pb2.FilterCreateActionRequest(filter=self.data, data=action),
            timeout=Cuebot.Timeout).action)

    def getActions(self):
        """Returns the filter actions.

        :rtype:  list<opencue.wrappers.filter.Action>
        :return: list of the filter actions
        """
        response = self.stub.GetActions(filter_pb2.FilterGetActionsRequest(filter=self.data),
                                        timeout=Cuebot.Timeout)
        return [Action(action) for action in response.actions.actions]

    def getMatchers(self):
        """Returns the filter matchers.

        :rtype:  list<opencue.wrapper.filter.Matcher>
        :return: list of the filter matchers
        """
        response = self.stub.GetMatchers(filter_pb2.FilterGetMatchersRequest(filter=self.data),
                                         timeout=Cuebot.Timeout)
        return [Matcher(matcher) for matcher in response.matchers.matchers]

    def lowerOrder(self):
        """Lowers the order of the filter relative to other filters."""
        self.stub.LowerOrder(filter_pb2.FilterLowerOrderRequest(filter=self.data),
                             timeout=Cuebot.Timeout)

    def raiseOrder(self):
        """Raises the order of the filter relative to other filters."""
        self.stub.RaiseOrder(filter_pb2.FilterRaiseOrderRequest(filter=self.data),
                             timeout=Cuebot.Timeout)

    def orderFirst(self):
        """Orders the filter above all other filters."""
        self.stub.OrderFirst(filter_pb2.FilterOrderFirstRequest(filter=self.data),
                             timeout=Cuebot.Timeout)

    def orderLast(self):
        """Orders the filter below all other filters."""
        self.stub.OrderLast(filter_pb2.FilterOrderLastRequest(filter=self.data),
                            timeout=Cuebot.Timeout)

    def runFilterOnGroup(self, group):
        """Runs the filter on the group provided.

        :type  group: opencue.wrapper.group.Group
        :param group: group to run the filter on
        """
        self.stub.RunFilterOnGroup(
            filter_pb2.FilterRunFilterOnGroupRequest(filter=self.data, group=group.data),
            timeout=Cuebot.Timeout)

    def runFilterOnJobs(self, jobs):
        """Runs the filter on the list of jobs provided.

        :type  jobs: list<opencue.wrapper.job.Job>
        :param jobs: jobs to run the filter on
        """
        jobSeq = job_pb2.JobSeq(jobs=[job.data for job in jobs])
        self.stub.RunFilterOnJobs(
            filter_pb2.FilterRunFilterOnJobsRequest(filter=self.data, jobs=jobSeq),
            timeout=Cuebot.Timeout)

    def setEnabled(self, value):
        """Enables or disables the filter.

        :type  value: bool
        :param value: true to enable the filter and false to disable it
        """
        self.stub.SetEnabled(filter_pb2.FilterSetEnabledRequest(filter=self.data, enabled=value),
                             timeout=Cuebot.Timeout)

    def setName(self, name):
        """Sets the name of the filter.

        :type  name: str
        :param name: new filter name
        """
        self.stub.SetName(filter_pb2.FilterSetNameRequest(filter=self.data, name=name),
                          timeout=Cuebot.Timeout)

    def setType(self, filterType):
        """Changes the filter type.

        :type  filterType: opencue_proto.filter_pb2.FilterType
        :param filterType: the new filter type
        """
        self.stub.SetType(filter_pb2.FilterSetTypeRequest(filter=self.data, type=filterType),
                          timeout=Cuebot.Timeout)

    def setOrder(self, order):
        """Directly sets the order of the filter.

        :type  order: int
        :param order: the new filter order
        """
        self.stub.SetOrder(filter_pb2.FilterSetOrderRequest(filter=self.data, order=order),
                           timeout=Cuebot.Timeout)

    def name(self):
        """Returns the filter name.

        :rtype:  str
        :return: the filter name
        """
        return self.data.name

    def type(self):
        """Returns the filter type.

        :rtype:  opencue_proto.filter_pb2.FilterType
        :return: the filter type
        """
        return self.data.type

    def order(self):
        """Returns the current position of the filter.

        :rtype:  float
        :return: the current position of the filter"""
        return self.data.order

    def isEnabled(self):
        """Returns whether the filter is enabled.

        :rtype:  bool
        :return: whether the filter is enabled
        """
        return self.data.enabled

    def id(self):
        """Returns the id of the filter.

        :rtype:  str
        :return: id of the filter
        """
        return self.data.id


class Action(object):
    """This class contains the grpc implementation related to an Action."""

    class ActionType(enum.IntEnum):
        """Enum representing the type of Action to be performed."""
        MOVE_JOB_TO_GROUP = filter_pb2.MOVE_JOB_TO_GROUP
        PAUSE_JOB = filter_pb2.PAUSE_JOB
        SET_JOB_MIN_CORES = filter_pb2.SET_JOB_MIN_CORES
        SET_JOB_MAX_CORES = filter_pb2.SET_JOB_MAX_CORES
        STOP_PROCESSING = filter_pb2.STOP_PROCESSING
        SET_JOB_PRIORITY = filter_pb2.SET_JOB_PRIORITY
        SET_ALL_RENDER_LAYER_TAGS = filter_pb2.SET_ALL_RENDER_LAYER_TAGS
        SET_ALL_RENDER_LAYER_MEMORY = filter_pb2.SET_ALL_RENDER_LAYER_MEMORY
        SET_ALL_RENDER_LAYER_MIN_CORES = filter_pb2.SET_ALL_RENDER_LAYER_MIN_CORES
        SET_ALL_RENDER_LAYER_MAX_CORES = filter_pb2.SET_ALL_RENDER_LAYER_MAX_CORES
        SET_MEMORY_OPTIMIZER = filter_pb2.SET_MEMORY_OPTIMIZER

    class ActionValueType(enum.IntEnum):
        """Enum representing the type of the action's object."""
        GROUP_TYPE = filter_pb2.GROUP_TYPE
        STRING_TYPE = filter_pb2.STRING_TYPE
        INTEGER_TYPE = filter_pb2.INTEGER_TYPE
        FLOAT_TYPE = filter_pb2.FLOAT_TYPE
        BOOLEAN_TYPE = filter_pb2.BOOLEAN_TYPE
        NONE_TYPE = filter_pb2.NONE_TYPE

    def __init__(self, action=None):
        self.data = action
        self.stub = Cuebot.getStub('action')

    def getParentFilter(self):
        """Returns the filter the action belongs to.

        :rtype: opencue.wrappers.filter.Filter
        :return: the filter the action belongs to
        """
        response = self.stub.GetParentFilter(
            filter_pb2.ActionGetParentFilterRequest(action=self.data),
            timeout=Cuebot.Timeout)
        return Filter(response.filter)

    def delete(self):
        """Deletes the action."""
        self.stub.Delete(filter_pb2.ActionDeleteRequest(action=self.data), timeout=Cuebot.Timeout)

    def commit(self):
        """Commits any changes to the action to the database."""
        if self.isNew():
            raise Exception(
                "unable to commit action that has not been created, proxy does not exist")
        self.stub.Commit(filter_pb2.ActionCommitRequest(action=self.data), timeout=Cuebot.Timeout)

    def isNew(self):
        """Returns whether the action has been initialized yet.

        :rtype: bool
        :return: True if the action has been initialized with data from the database
        """
        return self.data is None

    def name(self):
        """Returns the name of the action.

        :rtype: str
        :return: name of the action
        """
        if self.value() is None:
            return "%s" % ActionType.Name(self.type())
        return "%s %s" % (ActionType.Name(self.type()), self.value())

    def value(self):
        """Returns the value of the action; what will happen if the filter is matched.

        Type of value returned depends on the action's value_type.

        :rtype: str/int/float/bool
        :return: value of the action
        """
        valueType = self.data.value_type
        if valueType == filter_pb2.GROUP_TYPE:
            return self.data.group_value
        if valueType == filter_pb2.STRING_TYPE:
            return self.data.string_value
        if valueType == filter_pb2.INTEGER_TYPE:
            return self.data.integer_value
        if valueType == filter_pb2.FLOAT_TYPE:
            return self.data.float_value
        if valueType == filter_pb2.BOOLEAN_TYPE:
            return self.data.boolean_value
        return None

    def type(self):
        """Returns the type of the action.

        An action's type determines what will happen if the action is triggered by its filter.
        For example, if the type is GROUP_TYPE, the object which has triggered the filter will
        be assigned to the group specified by the action's value.

        :rtype: filter_pb2.ActionValueType
        :return: the type of the action
        """
        return self.data.type

    def setTypeAndValue(self, actionType, value):
        """Sets a new type and value for the action.

        These fields should be set together as the value can be only be properly validated and
        stored in the correct database field if the type is also known.

        :type  actionType: filter_pb2.ActionValueType
        :param actionType: the new type of the action
        :type  value: str/int/float/bool
        :param value: the new value of the action
        """
        self.data.type = actionType
        if actionType == filter_pb2.MOVE_JOB_TO_GROUP:
            if not isinstance(value, job_pb2.Group):
                raise TypeError("invalid group argument, not a group")
            if not value.id:
                raise ValueError("group is not a valid rpc object")
            self.data.group_value = value.id
            self.data.value_type = filter_pb2.GROUP_TYPE

        elif actionType in (filter_pb2.PAUSE_JOB, filter_pb2.SET_MEMORY_OPTIMIZER):
            self.data.boolean_value = value
            self.data.value_type = filter_pb2.BOOLEAN_TYPE

        elif actionType in (filter_pb2.SET_JOB_PRIORITY,
                            filter_pb2.SET_ALL_RENDER_LAYER_MEMORY):
            self.data.integer_value = int(value)
            self.data.value_type = filter_pb2.INTEGER_TYPE

        elif actionType in (filter_pb2.SET_JOB_MIN_CORES,
                            filter_pb2.SET_JOB_MAX_CORES,
                            filter_pb2.SET_ALL_RENDER_LAYER_MIN_CORES,
                            filter_pb2.SET_ALL_RENDER_LAYER_MAX_CORES):
            self.data.float_value = float(value)
            self.data.value_type = filter_pb2.FLOAT_TYPE

        elif actionType == filter_pb2.SET_ALL_RENDER_LAYER_TAGS:
            self.data.string_value = value
            self.data.value_type = filter_pb2.STRING_TYPE

        elif actionType == filter_pb2.STOP_PROCESSING:
            self.data.value_type = filter_pb2.NONE_TYPE

        else:
            raise Exception("invalid action type: %s" % actionType)

        self.commit()

    def id(self):
        """Returns the id of the action.

        :rtype:  str
        :return: id of the action
        """
        return self.data.id


class Matcher(object):
    """This class contains the grpc implementation related to a Matcher.

    Matchers belong to a single filter, and indicate the conditions where a given object will
    satisfy that filter, i.e. if it will trigger the actions in that filter.
    """

    class MatchSubject(enum.IntEnum):
        """Enum representing the type of the subject; the thing being matched."""
        JOB_NAME = filter_pb2.JOB_NAME
        SHOW = filter_pb2.SHOW
        SHOT = filter_pb2.SHOT
        USER = filter_pb2.USER
        SERVICE_NAME = filter_pb2.SERVICE_NAME
        PRIORITY = filter_pb2.PRIORITY
        FACILITY = filter_pb2.FACILITY
        LAYER_NAME = filter_pb2.LAYER_NAME

    class MatchType(enum.IntEnum):
        """Enum representing the type of matching that will occur."""
        CONTAINS = filter_pb2.CONTAINS
        DOES_NOT_CONTAIN = filter_pb2.DOES_NOT_CONTAIN
        IS = filter_pb2.IS
        IS_NOT = filter_pb2.IS_NOT
        REGEX = filter_pb2.REGEX
        BEGINS_WITH = filter_pb2.BEGINS_WITH
        ENDS_WITH = filter_pb2.ENDS_WITH

    def __init__(self, matcher=None):
        self.data = matcher
        self.stub = Cuebot.getStub('matcher')

    def getParentFilter(self):
        """Returns the filter the matcher belongs to.

        :rtype: opencue.wrappers.filter.Filter
        :return: the filter the matcher belongs to
        """
        response = self.stub.GetParentFilter(
            filter_pb2.MatcherGetParentFilterRequest(matcher=self.data),
            timeout=Cuebot.Timeout)
        return Filter(response.filter)

    def delete(self):
        """Deletes the matcher."""
        self.stub.Delete(filter_pb2.MatcherDeleteRequest(matcher=self.data), timeout=Cuebot.Timeout)

    def commit(self):
        """Commits any changes to the matcher to the database."""
        if self.isNew():
            raise Exception(
                "unable to commit matcher that has not been created, proxy does not exist")
        self.data.input = self.data.input.replace(" ", "")
        self.stub.Commit(filter_pb2.MatcherCommitRequest(matcher=self.data), timeout=Cuebot.Timeout)

    def isNew(self):
        """Returns whether the matcher has been initialized yet with data from the database.

        :rtype: bool
        :return: True if the matcher has been initialized
        """
        return self.data is None

    def name(self):
        """Returns the name of the matcher.

        :rtype: str
        :return: the name of the matcher
        """
        return "%s %s %s" % (
            MatchSubject.Name(self.data.subject), MatchType.Name(self.data.type), self.data.input)

    def subject(self):
        """Returns the subject of the matcher; the type of object to be matched.

        :rtype: filter_pb2.MatchSubject
        :return: the subject of the matcher
        """
        return self.data.subject

    def type(self):
        """Returns the type of the matcher; the kind of comparison used to determine a match.

        :rtype: filter_pb2.MatchType
        :return: the type of the matcher
        """
        return self.data.type

    def input(self):
        """Returns the input data of the matcher; the value to be matched against.

        :rtype: str
        :return: input data of the matcher
        """
        return self.data.input

    def id(self):
        """Returns the id of the matcher.

        :rtype:  str
        :return: id of the matcher
        """
        return self.data.id

    def setSubject(self, value):
        """Sets a new subject for the matcher.

        :type  value: str
        :param value: new subject for the matcher
        """
        self.data.subject = value
        self.commit()

    def setType(self, value):
        """Sets a new type for the matcher.

        :type  value: filter_pb2.MatchType
        :param value: new type for the matcher
        """
        self.data.type = value
        self.commit()

    def setInput(self, value):
        """Set new input data for the matcher.

        :type  value: str
        :param value: new input data for the matcher
        """
        value = value.replace(" ", "")
        self.data.input = str(value)
        self.commit()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to hosts."""

import enum
import os
import time
from builtins import input
import grpc

from opencue_proto import comment_pb2
from opencue_proto import host_pb2
from opencue import Cuebot
from opencue import util
from opencue import search
import opencue.wrappers.comment
# pylint: disable=cyclic-import
import opencue.wrappers.proc
import opencue.wrappers.render_partition


class Host(object):
    """This class contains the grpc implementation related to a Host."""

    class HardwareState(enum.IntEnum):
        """Enum representing the hardware state of the host."""
        UP = host_pb2.UP
        DOWN = host_pb2.DOWN
        REBOOTING = host_pb2.REBOOTING
        REBOOT_WHEN_IDLE = host_pb2.REBOOT_WHEN_IDLE
        REPAIR = host_pb2.REPAIR

    class HostTagType(enum.IntEnum):
        """Enum representing the type of a host tag."""
        MANUAL = host_pb2.MANUAL
        HARDWARE = host_pb2.HARDWARE
        ALLOC = host_pb2.ALLOC
        HOSTNAME = host_pb2.HOSTNAME

    class LockState(enum.IntEnum):
        """Enum representing whether the host is locked."""
        OPEN = host_pb2.OPEN
        LOCKED = host_pb2.LOCKED
        NIMBY_LOCKED = host_pb2.NIMBY_LOCKED

    class ThreadMode(enum.IntEnum):
        """Enum representing the thread mode of the host."""
        AUTO = host_pb2.AUTO
        ALL = host_pb2.ALL
        VARIABLE = host_pb2.VARIABLE

    def __init__(self, host=None):
        self.data = host
        self.__id = host.id
        self.stub = Cuebot.getStub('host')

    def lock(self):
        """Locks the host so that it no longer accepts new frames"""
        # Update the cached lock_state.
        self.data.lock_state = self.LockState.LOCKED
        self.stub.Lock(host_pb2.HostLockRequest(host=self.data), timeout=Cuebot.Timeout)

    def unlock(self):
        """Unlocks the host.

        Cancels any actions that were waiting for all running frames to finish."""
        # Update the cached lock_state.
        self.data.lock_state = self.LockState.OPEN
        self.stub.Unlock(host_pb2.HostUnlockRequest(host=self.data), timeout=Cuebot.Timeout)

    def delete(self):
        """Deletes the host from the cuebot"""
        self.stub.Delete(host_pb2.HostDeleteRequest(host=self.data), timeout=Cuebot.Timeout)

    def getProcs(self):
        """Returns a list of procs under this host.

        :rtype: list<opencue.wrappers.proc.Proc>
        :return: A list of procs under this host
        """
        response = self.stub.GetProcs(host_pb2.HostGetProcsRequest(host=self.data),
                                      timeout=Cuebot.Timeout)
        return [opencue.wrappers.proc.Proc(p) for p in response.procs.procs]

    def redirectToJob(self, procs, job):
        """Unbooks and redirects the proc to the specified job.  Optionally
        kills the proc immediately.

        :param procs: list<opencue.wrappers.proc.Proc>
        :param job: job id
        """
        self.stub.RedirectToJob(
            host_pb2.HostRedirectToJobRequest(host=self.data,
                                              proc_names=[proc.data.id for proc in procs],
                                              job_id=job.data.id), timeout=Cuebot.Timeout)

    def getRenderPartitions(self):
        """Returns a list of render partitions associated with this host

        :rtype:  list<renderPartition_pb2.RenderPartition>
        :return: list of render partitions under this host
        """
        response = self.stub.GetRenderPartitions(host_pb2.HostGetRenderPartitionsRequest(
            host=self.data), timeout=Cuebot.Timeout)
        return [opencue.wrappers.render_partition.RenderPartition(p)
            for p in response.render_partitions.render_partitions]

    def rebootWhenIdle(self):
        """Sets the machine to reboot once idle.

        The host will no longer accept new frames."""
        self.stub.RebootWhenIdle(host_pb2.HostRebootWhenIdleRequest(host=self.data),
                                 timeout=Cuebot.Timeout)

    def reboot(self):
        """Causes the host to kill all running frames and reboot the machine."""
        self.stub.Reboot(host_pb2.HostRebootRequest(host=self.data), timeout=Cuebot.Timeout)

    def addTags(self, tags):
        """Adds tags to a host.

        :type  tags: list<str>
        :param tags: The tags to add
        """
        # Filter out duplicates
        tags = [item for item in tags if item not in self.data.tags]
        # Update the cached tags.
        self.data.tags.extend(tags)
        self.stub.AddTags(host_pb2.HostAddTagsRequest(host=self.data, tags=tags),
                          timeout=Cuebot.Timeout)

    def removeTags(self, tags):
        """Removes tags from this host.

        :type  tags: list<str>
        :param tags: The tags to remove
        """
        self.stub.RemoveTags(host_pb2.HostRemoveTagsRequest(host=self.data, tags=tags),
                             timeout=Cuebot.Timeout)

    def renameTag(self, oldTag, newTag):
        """Renames a tag.

        :type  oldTag: str
        :param oldTag: old tag to rename
        :type  newTag: str
        :param newTag: new name for the tag
        """
        self.stub.RenameTag(
            host_pb2.HostRenameTagRequest(host=self.data, old_tag=oldTag, new_tag=newTag),
            timeout=Cuebot.Timeout)

    @util.grpcExceptionParser
    def setAllocation(self, allocation):
        """Sets the host to the given allocation.

        :type  allocation: opencue.wrappers.allocation.Allocation
        :param allocation: allocation to put the host under
        """
        self.stub.SetAllocation(
            host_pb2.HostSetAllocationRequest(host=self.data, allocation_id=allocation.id()),
            timeout=Cuebot.Timeout)

    def addComment(self, subject, message):
        """Appends a comment to the host's comment list.

        :type  subject: str
        :param subject: Subject data
        :type  message: str
        :param message: Message data
        """
        comment = comment_pb2.Comment(
            user=os.getenv("USER", "unknown"),
            subject=subject,
            message=message or " ",
            timestamp=0
        )
        self.stub.AddComment(host_pb2.HostAddCommentRequest(host=self.data, new_comment=comment),
                             timeout=Cuebot.Timeout)

    def getComments(self):
        """Returns the host's comment list.

        :rtype:  list<opencue.wrappers.comment.Comment>
        :return: the comment list of the host
        """
        response = self.stub.GetComments(host_pb2.HostGetCommentsRequest(host=self.data),
                                         timeout=Cuebot.Timeout)
        commentSeq = response.comments
        return [opencue.wrappers.comment.Comment(c) for c in commentSeq.comments]

    def setHardwareState(self, state):
        """Sets the host hardware state.

        :type  state: host_pb2.HardwareState
        :param state: state to set host to
        """
        self.stub.SetHardwareState(
            host_pb2.HostSetHardwareStateRequest(host=self.data, state=state),
            timeout=Cuebot.Timeout)

    def setOs(self, osName):
        """Sets the host operating system.

        :type  osName: string
        :param osName: os value to set host to
        """
        self.stub.SetOs(host_pb2.HostSetOsRequest(host=self.data, os=osName),
                        timeout=Cuebot.Timeout)

    def setThreadMode(self, mode):
        """Sets the host thread mode.

        :type  mode: host_pb2.ThreadMode
        :param mode: ThreadMode to set host to
        """
        self.stub.SetThreadMode(host_pb2.HostSetThreadModeRequest(host=self.data, mode=mode),
                                timeout=Cuebot.Timeout)

    @staticmethod
    def hasHostRebootedSince(host, start_time):
        """
        Returns whether a host has rebooted since `start_time` or is in process of
        rebooting (`state = REBOOT_WHEN_IDLE`)

        :param host: Host wrapper
        :param start_time: epoch time
        :return: True if host booted after start_time or is rebooting
        """
        return (host.state() == host_pb2.HardwareState.Value('REBOOT_WHEN_IDLE') or
                (host.state() == host_pb2.HardwareState.Value('UP') and
                 host.bootTime() < start_time))

    @staticmethod
    def rebootFarmSafely(group_size, start_time=None, **options):
        """Requests an idle reboot for nodes found using the
        options search criteria. For safety, workstations are always excluded.

        Method can only be called interactively

        For example::
        from opencue.wrappers.host import Host
        Host.rebootFarmSafely(5, 1, alloc=["lax.ngp"])

        Uses the hostSearch module to find hosts and requests a reboot for each
        of them in groups defined by the arg `group_size`. If `start_time` is
        provided the function will only target hosts that have
        `boot_time` > `start_time`.

        Possible hostSearch args:
           - host: host names - list
           - match: host name substring match - str
           - regex: a host name search by regular expression - str
           - id: a search by unique id - str
           - alloc: search by allocation. - list

        :param group_size: Reboot hosts in groups limited to this size
        :param start_time: if not none, only hosts with boot_time<start_time will be targeted
        :param options: HostSearch params
        """
        hosts = search.HostSearch.byOptions(**options)
        # Workstations are marked as Nimby, ignore those.
        hosts = [host for host in hosts if not host.isNimbyEnabled() and host.isUp()]
        check_hosts_interval_seconds = 30

        if len(hosts) == 0:
            print("No hosts found")
            return

        print("Rebooting hosts:\n%s.\n\n"
              "Are you sure hosts on this list are safe to be rebooted? [Y/n]" %
              [str(host.name()) for host in hosts])
        while True:
            choice = input()
            if choice == "Y":
                break
            if choice == "n":
                return

        groups = [hosts[x:x+group_size] for x in range(0, len(hosts), group_size)]

        host_still_rebooting = []

        if start_time is None:
            start_time = int(time.time())
        for group in groups:
            group_host_names = []
            for host in group:
                if host.bootTime() < start_time:
                    try:
                        host.rebootWhenIdle()
                    except grpc.RpcError as rpc_error:
                        # pylint: disable=no-member
                        if rpc_error.code() in [grpc.StatusCode.UNAVAILABLE,
                                                grpc.StatusCode.CANCELLED]:
                            # Wait and Retry
                            time.sleep(5)
                            host.rebootWhenIdle()
                        else:
                            raise rpc_error
                        # pylint: enable=no-member

                group_host_names.append(str(host.name()))
            print("Requesting reboot for %s. Waiting for completion." % group_host_names)

            # Wait until 80% of a group gets upgraded to jump to the next group
            while True:
                try:
                    hosts = search.HostSearch.byName(group_host_names)
                    rebooting_hosts = [str(host.name()) for host in hosts
                                       if Host.hasHostRebootedSince(host, start_time)]
                    if len(rebooting_hosts) <= len(hosts) * 0.2:
                        host_still_rebooting.extend(rebooting_hosts)
                        print("Moving to the next group, left behind %s..\n" % rebooting_hosts)
                        break
                    print("Still waiting on %s.." % rebooting_hosts)
                    time.sleep(check_hosts_interval_seconds)
                except grpc.RpcError as rpc_error:
                    # Ignore rpc unavailable to survive short service outages on the server side
                    # pylint: disable=no-member
                    if rpc_error.code() in [grpc.StatusCode.UNAVAILABLE, grpc.StatusCode.CANCELLED]:
                        continue
                    # pylint: enable=no-member

        # Wait for remaining hosts to finish (20% not awaited for)
        if host_still_rebooting:
            print("Waiting on remaining hosts to reboot")
            while True:
                try:
                    hosts = search.HostSearch.byName(host_still_rebooting)
                    rebooting_hosts = [str(host.name()) for host in hosts
                                       if Host.hasHostRebootedSince(host, start_time)]
                    if len(rebooting_hosts) == 0:
                        break
                    host_still_rebooting = rebooting_hosts
                    print("Still waiting on %s.." % rebooting_hosts)
                    time.sleep(check_hosts_interval_seconds)
                except grpc.RpcError as rpc_error:
                    # Ignore rpc unavailable to survive short service outages on the server side
                    # pylint: disable=no-member
                    if rpc_error.code() in [grpc.StatusCode.UNAVAILABLE, grpc.StatusCode.CANCELLED]:
                        continue
                    # pylint: enable=no-member
        print("Finished rebooting requested hosts")

    @staticmethod
    def monitorRebootFarm(start_time, **options):
        """Monitor hosts rebooting

        Possible args:
           - host: host names - list
           - match: host name substring match - str
           - regex: a host name search by regular expression - str
           - id: a search by unique id - str
           - alloc: search by allocation. - list

        :param start_time: if not none only hosts with boot_time<start_time will be targeted
        :param options: HostSearch params
        """
        hosts = search.HostSearch.byOptions(**options)
        # Workstations are marked as Nimby, ignore those.
        hosts_names = [host.name() for host in hosts if not host.isNimbyEnabled() and host.isUp()]
        check_hosts_interval_seconds = 30

        if len(hosts) == 0:
            print("No hosts found")
            return

        if start_time is None:
            start_time = int(time.time())

        # Wait until 80% of a group gets upgraded to jump to the next group
        while True:
            try:
                hosts = search.HostSearch.byName(hosts_names)
                rebooting_hosts = [str(host.name()) for host in hosts
                                   if Host.hasHostRebootedSince(host, start_time)]
                if len(rebooting_hosts) == 0:
                    break
                print("Still waiting on %s hosts: %s.." % (len(rebooting_hosts), rebooting_hosts))
                time.sleep(check_hosts_interval_seconds)
            except grpc.RpcError as rpc_error:
                # Ignore rpc unavailable to survive short service outages on the server side
                # pylint: disable=no-member
                if rpc_error.code() in [grpc.StatusCode.UNAVAILABLE, grpc.StatusCode.CANCELLED]:
                    continue
                # pylint: enable=no-member

        print("Finished rebooting hosts")

    def id(self):
        """Returns the id of the host.

        :rtype:  str
        :return: id of the host
        """
        if not hasattr(self, "__id"):
            self.__id = self.data.id
        return self.__id

    def name(self):
        """Returns the name of the host.

        :rtype:  str
        :return: name of the host
        """
        return self.data.name

    def isNimbyEnabled(self):
        """Returns true if nimby is enabled.

        :rtype:  bool
        :return: True if nimby is enabled
        """
        return self.data.nimby_enabled

    def isUp(self):
        """Returns True if the host hardware state indicates the machine is up.

        :rtype:  bool
        :return: True if the host is up
        """
        return self.data.state == host_pb2.HardwareState.Value('UP')

    def isLocked(self):
        """Returns True if the host is locked.

        :rtype:  bool
        :return: True if the host is locked
        """
        return self.data.lock_state == host_pb2.LockState.Value('LOCKED')

    def isCommented(self):
        """Returns true if the host has a comment.

        :rtype:  bool
        :return: whether the host has a comment
        """
        return self.data.has_comment

    def cores(self):
        """Returns the total number of cores the host has.

        :rtype: float
        :return: total number of host cores
        """
        return self.data.cores

    def coresReserved(self):
        """Returns the number of cores the host has which are currently reserved.

        :rtype: float
        :return: number of cores reserved
        """
        return self.data.cores - self.data.idle_cores

    def coresIdle(self):
        """Returns the number of cores the host currently has idel.

        :rtype: float
        :return: number of cores idle
        """
        return self.data.idle_cores

    def mem(self):
        """Returns the amount of memory the host has in kb.

        :rtype:  int
        :return: amount of memory in kb
        """
        return self.data.memory

    def memReserved(self):
        """Returns the amount of memory the host has currently reserved.

        :rtype:  int
        :return: amount of memory reserved in kb
        """
        return self.data.memory - self.data.idle_memory

    def memIdle(self):
        """Returns the amount of memory the host currently has idle.

        :rtype:  int
        :return: amount of idle memory in kb
        """
        return self.data.idle_memory

    def memUsed(self):
        """Returns the amount of memory the host currently has in use.

        :rtype:  int
        :return: amount of in-use memory in kb
        """
        return self.data.total_memory - self.data.free_memory

    def memTotal(self):
        """Returns the total amount of memory the host has.

        :rtype:  int
        :return: total amount of memory on host
        """
        return self.data.total_memory

    def memFree(self):
        """Returns the amount of memory the host currently has free.

        :rtype:  int
        :return: amount of free memory in kb
        """
        return self.data.free_memory

    def swapUsed(self):
        """Returns the amount of swap space the host has in use.

        :rtype:  int
        :return: amount of swap used in kb
        """
        return self.data.total_swap - self.data.free_swap

    def swapTotal(self):
        """Returns the total amount of swap space the host has.

        :rtype:  int
        :return: total amount of swap space in kb
        """
        return self.data.total_swap

    def swapFree(self):
        """Returns the amount of free swap space the host has.

        :rtype:  int
        :return: amount of free swap space in kb
        """
        return self.data.free_swap

    def mcpUsed(self):
        """Returns the amount of /mcp space the host is using.

        :rtype:  int
        :return: amount of mcp used in kb
        """
        return self.mcpTotal() - self.mcpFree()

    def mcpTotal(self):
        """Returns the total amount of /mcp space the host has.

        :rtype:  int
        :return: total amount of mcp in kb
        """
        return self.data.total_mcp

    def mcpFree(self):
        """Returns the amount of free /mcp space the host has.

        :rtype:  int
        :return: amount of mcp free in kb
        """
        return self.data.free_mcp

    def load(self):
        """Returns the host load average.

        :rtype:  int
        :return: host load average * 100
        """
        return self.data.load

    def bootTime(self):
        """Returns the time the host was booted.

        :rtype:  int
        :return: host boot time as an epoch
        """
        return self.data.boot_time

    def pingTime(self):
        """Returns the last time the host sent a status report.

        :rtype:  int
        :return: last ping time as an epoch
        """
        return self.data.ping_time

    def pingLast(self):
        """Returns the number of seconds since the last time the host sent a status report.

        :rtype:  int
        :return: seconds since last ping
        """
        return int(time.time() - self.pingTime())

    def tags(self):
        """Returns the tags the host has.

        :rtype:  list<str>
        :return: list of tags applied to the host
        """
        return self.data.tags

    def state(self):
        """Returns the hardware state of the host.

        :rtype:  host_pb2.HardwareState
        :return: the hardware state of the host
        """
        return self.data.state

    def lockState(self):
        """Returns the lock state of the host.

        :rtype:  host_pb2.LockState
        :return: the lock state of the host
        """
        return self.data.lock_state

    def os(self):
        """
        :rtype: str
        :return: the operating system of the host
        """
        return self.data.os

class NestedHost(Host):
    """This class contains information and actions related to a nested host."""

    def __init__(self, host):
        super(NestedHost, self).__init__(host)

    def children(self):
        """The procs running on this host.

        :rtype:  host_pb2.NestedProcSeq
        :return: the procs running on this host
        """
        return self.data.procs



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Modules for classes related to groups."""

from opencue_proto import job_pb2
from opencue import Cuebot
import opencue.wrappers.job


class Group(object):
    """This class contains the grpc implementation related to a Group."""

    def __init__(self, group=None):
        self.data = group
        self.stub = Cuebot.getStub('group')

    def createSubGroup(self, name):
        """Creates a subgroup under this group.

        :type  name: str
        :param name: name of the new subgroup
        """
        return Group(self.stub.CreateSubGroup(
            job_pb2.GroupCreateSubGroupRequest(group=self.data, name=name),
            timeout=Cuebot.Timeout).group)

    def delete(self):
        """Deletes the group."""
        self.stub.Delete(job_pb2.GroupDeleteRequest(group=self.data), timeout=Cuebot.Timeout)

    def setName(self, name):
        """Sets the name of the group.

        :type  name: str
        :param name: new name for the group
        """
        self.stub.SetName(job_pb2.GroupSetNameRequest(group=self.data, name=name),
                          timeout=Cuebot.Timeout)

    def setMaxCores(self, value):
        """Sets the maximum cores of everything in the group.

        :type  value: int
        :param value: new maximum number of cores
        """
        self.stub.SetMaxCores(job_pb2.GroupSetMaxCoresRequest(group=self.data, max_cores=value),
                              timeout=Cuebot.Timeout)

    def setMinCores(self, value):
        """Sets the minimum cores of everything the group.

        :type  value: int
        :param value: new minimum number of cores
        """
        self.stub.SetMinCores(job_pb2.GroupSetMinCoresRequest(group=self.data, min_cores=value),
                              timeout=Cuebot.Timeout)

    def setMaxGpus(self, value):
        """Sets the maximum gpus of everything in the group.

        :type  value: int
        :param value: new maximum number of gpus
        """
        self.stub.SetMaxGpus(job_pb2.GroupSetMaxGpusRequest(group=self.data, max_gpus=value),
                             timeout=Cuebot.Timeout)

    def setMinGpus(self, value):
        """Sets the minimum gpus of everything the group.

        :type  value: int
        :param value: new minimum number of gpus
        """
        self.stub.SetMinGpus(job_pb2.GroupSetMinGpusRequest(group=self.data, min_gpus=value),
                             timeout=Cuebot.Timeout)

    def setDefaultJobPriority(self, value):
        """Sets the default job priority for everything in the group.

        :type  value: int
        :param value: new default priority
        """
        self.stub.SetDefaultJobPriority(
            job_pb2.GroupSetDefJobPriorityRequest(group=self.data, priority=value),
            timeout=Cuebot.Timeout)

    def setDefaultJobMinCores(self, value):
        """Sets the default job minimum cores for everything in the group.

        :type  value: int
        :param value: new default job minimum cores
        """
        self.stub.SetDefaultJobMinCores(
            job_pb2.GroupSetDefJobMinCoresRequest(group=self.data, min_cores=value),
            timeout=Cuebot.Timeout)

    def setDefaultJobMaxCores(self, value):
        """Sets the default job maximum cores for everything in the group.

        :type  value: int
        :param value: new default job maximum cores
        """
        self.stub.SetDefaultJobMaxCores(
            job_pb2.GroupSetDefJobMaxCoresRequest(group=self.data, max_cores=value),
            timeout=Cuebot.Timeout)

    def setDefaultJobMinGpus(self, value):
        """Sets the default job minimum gpus for everything in the group.

        :type  value: int
        :param value: new default job minimum gpus
        """
        self.stub.SetDefaultJobMinGpus(
            job_pb2.GroupSetDefJobMinGpusRequest(group=self.data, min_gpus=value),
            timeout=Cuebot.Timeout)

    def setDefaultJobMaxGpus(self, value):
        """Sets the default job maximum gpus for everything in the group.

        :type  value: int
        :param value: new default job maximum gpus
        """
        self.stub.SetDefaultJobMaxGpus(
            job_pb2.GroupSetDefJobMaxGpusRequest(group=self.data, max_gpus=value),
            timeout=Cuebot.Timeout)

    def getGroups(self):
        """Returns child groups of this group.

        :rtype: list<opencue.wrappers.group.Group>
        :return: list of child groups
        """
        response = self.stub.GetGroups(job_pb2.GroupGetGroupsRequest(group=self.data),
                                       timeout=Cuebot.Timeout)
        return [Group(g) for g in response.groups.groups]

    def getJobs(self):
        """Returns the jobs in this group.

        :rtype:  list<opencue.wrappers.job.Job>
        :return: list of jobs in this group
        """
        response = self.stub.GetJobs(job_pb2.GroupGetJobsRequest(group=self.data),
                                     timeout=Cuebot.Timeout)
        return [opencue.wrappers.job.Job(j) for j in response.jobs.jobs]

    def reparentJobs(self, jobs):
        """Moves the given jobs into this group.

        :type  jobs: list<opencue.wrappers.job.Job>
        :param jobs: The jobs to add to this group
        """
        jobsToReparent = []
        for job in jobs:
            if isinstance(job, opencue.wrappers.job.NestedJob):
                job = job.asJob()
            jobsToReparent.append(job.data)
        jobSeq = job_pb2.JobSeq(jobs=jobsToReparent)
        self.stub.ReparentJobs(job_pb2.GroupReparentJobsRequest(group=self.data, jobs=jobSeq),
                               timeout=Cuebot.Timeout)

    def reparentGroups(self, groups):
        """Moves the given groups to be subgroups of this group.

        :type  groups: list<opencue.wrappers.group.Group>
        :param groups: the groups to make subgroups of this group
        """
        groupSeq = job_pb2.GroupSeq(groups=[group.data for group in groups])
        self.stub.ReparentGroups(
            job_pb2.GroupReparentGroupsRequest(group=self.data, groups=groupSeq),
            timeout=Cuebot.Timeout)

    def reparentGroupIds(self, groupIds):
        """Moves the given group ids into this group.

        :type  groupIds: list<str>
        :param groupIds: The group ids to make subgroups of this group
        """
        groups = [Group(job_pb2.Group(id=groupId)) for groupId in groupIds]
        self.reparentGroups(groups)

    def setDepartment(self, name):
        """Sets the group's department to the specified name.

        The department name must be one of the allowed department names.  All jobs in the group
        will inherit the new department name.

        :type  name: string
        :param name: a valid department name
        """
        self.stub.SetDepartment(job_pb2.GroupSetDeptRequest(group=self.data, dept=name),
                                timeout=Cuebot.Timeout)
        self.data.department = name

    def setGroup(self, parentGroup):
        """Sets this group's parent.

        :type  parentGroup: opencue.wrappers.group.Group
        :param parentGroup: group to parent under
        """
        self.stub.SetGroup(
            job_pb2.GroupSetGroupRequest(group=self.data, parent_group=parentGroup.data),
            timeout=Cuebot.Timeout)

    def id(self):
        """Returns the id of the group.

        :rtype:  str
        :return: id of the group
        """
        return self.data.id

    def name(self):
        """Returns the name of the group.

        :rtype:  str
        :return: name of the group"""
        return self.data.name

    def department(self):
        """Returns the department of the group.

        :rtype:  str
        :return: department of the group
        """
        return self.data.department

    def defaultJobPriority(self):
        """Returns the default job priority of the group.

        :rtype:  int
        :return: default job priority of the group
        """
        return self.data.default_job_priority

    def defaultJobMinCores(self):
        """Returns the default job minimum cores of the group.

        :rtype:  float
        :return: default job min cores
        """
        return self.data.default_job_min_cores

    def defaultJobMaxCores(self):
        """Returns the default job maximum cores of the group.

        :rtype:  float
        :return: default job max cores
        """
        return self.data.default_job_max_cores

    def maxCores(self):
        """Returns the maximum cores of the group.

        :rtype:  float
        :return: max cores of the group
        """
        return self.data.max_cores

    def minCores(self):
        """Returns the minimum cores of the group.

        :rtype:  float
        :return: min cores of the group
        """
        return self.data.min_cores

    def reservedCores(self):
        """Returns the total number of reserved cores for the group.

        :rtype:  float
        :return: total number of reserved cores
        """
        return self.data.group_stats.reserved_cores

    def totalRunning(self):
        """Returns the total number of running frames under this group.

        :rtype:  int
        :return: total number of running frames
        """
        return self.data.group_stats.running_frames

    def totalDead(self):
        """Returns the total number of dead frames under this group.

        :rtype:  int
        :return: total number of dead frames
        """
        return self.data.group_stats.dead_frames

    def totalPending(self):
        """Returns the total number of pending (dependent and waiting) frames under this group.

        :rtype:  int
        :return: total number of pending (dependent and waiting) frames
        """
        return self.data.group_stats.pending_frames

    def pendingJobs(self):
        """Returns the total number of running jobs.

        :rtype: int
        :return: total number of running jobs
        """
        return self.data.group_stats.pending_jobs


class NestedGroup(Group):
    """This class contains information and actions related to a nested group."""

    def __init__(self, group):
        super(NestedGroup, self).__init__(group)
        self.__children = []
        self.__children_init = False

    def createSubGroup(self, name):
        """Creates a subgroup under this nested group.

        :type  name: str
        :param name: name of the new subgroup
        """
        return self.asGroup().createSubGroup(name)

    def children(self):
        """Returns this group's jobs and child groups in a single array.

        :rtype: list<job_pb2.group/job_pb2.job>
        :return: list of all jobs and child groups in this group
        """
        if not self.__children_init:
            self.__children.extend(self.data.groups)
            self.__children.extend(self.data.jobs)
            self.__children_init = True
        return self.__children

    def asGroup(self):
        """Returns a Group object from this NestedGroup.

        :rtype: opencue.wrappers.group.Group
        :return: Group version of this NestedGroup
        """
        return Group(job_pb2.Group(
            id=self.data.id,
            name=self.data.name,
            department=self.data.department,
            default_job_priority=self.data.default_job_priority,
            default_job_min_cores=self.data.default_job_min_cores,
            default_job_max_cores=self.data.default_job_max_cores,
            min_cores=self.data.min_cores,
            max_cores=self.data.max_cores,
            level=self.data.level,
            group_stats=self.data.stats,
        ))

    def hasParent(self):
        """Whether this NestedGroup has a parent group.

        :rtype:  bool
        :return: whether the group has a parent group.
        """
        return self.data.HasField('parent')
